      subroutine read_flux(file_in,Nk,flux)
      implicit none
      include 'max.inc'
c     
c     Purpose: to read a flux file
c     
c     Input:
c       + file_in: file to read
c     
c     Output:
c       + Nk: number of wavenumber points
c       + flux: radiative flux
c         - flux(*,1): wavenumber [inv. cm]
c         - flux(*,2): radiative flux [W/m^2/inv. cm]
c     
c     I/O
      character*(Nchar_mx) file_in
      integer Nk
      double precision flux(1:Nk_mx,1:2)
c     temp
      integer ios,ik,nlines,j
      character*(Nchar_mx) nlines_file
c     label
      integer strlen
      character*(Nchar_mx) label
      label='subroutione read_flux'

      nlines_file='./nlines'
      call get_nlines(file_in,nlines_file,nlines)
      Nk=nlines-1
      if (Nk.gt.Nk_mx) then
         call error(label)
         write(*,*) 'Nk=',Nk
         write(*,*) 'while Nk_mx=',Nk_mx
         stop
      endif
      
      open(11,file=file_in(1:strlen(file_in))
     &     ,status='old',iostat=ios)
      if (ios.ne.0) then        ! file not found
         call error(label)
         write(*,*) 'File not found:'
         write(*,*) file_in(1:strlen(file_in))
         stop
      else
         write(*,*) 'Reading file: ',file_in(1:strlen(file_in))
         read(11,*)
         do ik=1,Nk
            read(11,*) (flux(ik,j),j=1,2)
         enddo                  ! ik
         write(*,*) 'done'
      endif
      close(11)

      return
      end


      subroutine read_total_probe_flux(file_in,pp,pa,pFup,pFdn,pFn)
      implicit none
      include 'max.inc'
c     
c     Purpose: to read a total probe flux file
c     
c     Input:
c       + file_in: file to read
c     
c     Output:
c       + Nlev: number of leves
c       + pp: probe pressure [atm]
c       + pa: probe altitude [m]
c       + pFup: probe upward flux [W/m^2]
c       + pFdn: probe downward flux [W/m^2]
c       + pFn: probe net flux [W/m^2]
c     
c     I/O
      character*(Nchar_mx) file_in
      double precision pp,pa,pFup,pFdn,pFn
c     temp
      integer ios
c     label
      integer strlen
      character*(Nchar_mx) label
      label='subroutione read_total_probe_flux'
      
      open(11,file=file_in(1:strlen(file_in))
     &     ,status='old',iostat=ios)
      if (ios.ne.0) then        ! file not found
         call error(label)
         write(*,*) 'File not found:'
         write(*,*) file_in(1:strlen(file_in))
         stop
      else
         read(11,*)
         read(11,*) pp,pa,pFup,pFdn,pFn
      endif
      close(11)

      return
      end



      subroutine read_flux_profile(file_in,Nlev,flux)
      implicit none
      include 'max.inc'
c     
c     Purpose: to read a flux file
c     
c     Input:
c       + file_in: file to read
c     
c     Output:
c       + Nlev: number of levels
c       + flux: radiative flux
c         - flux(*,1): pressure (atm)
c         - flux(*,2): spectrally integrated radiative flux [W/m^2]
c     
c     I/O
      character*(Nchar_mx) file_in
      integer Nlev
      double precision flux(1:Nlev_mx,1:2)
c     temp
      integer ios,ilev,nlines,j
      double precision raf
      character*(Nchar_mx) nlines_file
c     label
      integer strlen
      character*(Nchar_mx) label
      label='subroutione read_flux_profile'

      nlines_file='./nlines'
      call get_nlines(file_in,nlines_file,nlines)
      Nlev=nlines-1
      if (Nlev.gt.Nlev_mx) then
         call error(label)
         write(*,*) 'Nlev=',Nlev
         write(*,*) 'while Nlev_mx=',Nlev_mx
         stop
      endif
      
      open(11,file=file_in(1:strlen(file_in))
     &     ,status='old',iostat=ios)
      if (ios.ne.0) then        ! file not found
         call error(label)
         write(*,*) 'File not found:'
         write(*,*) file_in(1:strlen(file_in))
         stop
      else
         write(*,*) 'Reading file: ',file_in(1:strlen(file_in))
         read(11,*)
         do ilev=1,Nlev
            read(11,*) raf,(flux(ilev,j),j=1,2)
         enddo                  ! ik
         write(*,*) 'done'
      endif
      close(11)

      return
      end
      


      subroutine record_flux(file_out,header,Nk,flux)
      implicit none
      include 'max.inc'
c     
c     Purpose: to record a spectral signal to file
c     
c     Input:
c       + file_out: record file
c       + header: character string (let specify the variable name and unit)
c       + Nk: number of points
c
c     Output:
c       + flux: spectral signal to record (whatever the unit)
c
c     I/O
      character*(Nchar_mx) file_out
      character*(Nchar_mx) header
      integer Nk
      double precision flux(1:Nk_mx,1:2)
c     temp
      integer ik,j
c     label
      integer strlen
      character*(Nchar_mx) label
      label='subroutione record_flux'
      
      open(12,file=file_out(1:strlen(file_out)))
      write(12,'(a)') header(1:strlen(header))
      do ik=1,Nk
         write(12,*) (flux(ik,j),j=1,2)
      enddo                     ! ik
      close(12)
      
      return
      end
      

      subroutine record_flux_profile(file_out,header,Nlev,flux)
      implicit none
      include 'max.inc'
c     
c     Purpose: to record a flux profile to file
c     
c     Input:
c       + file_out: record file
c       + header: character string (let specify the variable name and unit)
c       + Nlev: number of atmospheric levels
c
c     Output:
c       + flux: flux profile to record (whatever the unit)
c
c     I/O
      character*(Nchar_mx) file_out
      character*(Nchar_mx) header
      integer Nlev
      double precision flux(1:Nlev_mx,1:2)
c     temp
      integer ilev,j
c     label
      integer strlen
      character*(Nchar_mx) label
      label='subroutione record_flux_profile'
      
      open(12,file=file_out(1:strlen(file_out)))
      write(12,'(a)') header(1:strlen(header))
      do ilev=1,Nlev
         write(12,*) (flux(ilev,j),j=1,2)
      enddo                     ! ik
      close(12)
      
      return
      end
      
