      program flux_differences
      implicit none
      include 'max.inc'
c     
c     Purpose: to compute the difference between two flux profiles (@TOA, @200hPa)
c
c     Variables
      character*(Nchar_mx) folder1,folder2
      character*(Nchar_mx) file_in1,file_in2
      character*(Nchar_mx) file_out
      double precision integral
c     label
      integer strlen
      character*(Nchar_mx) label
      label='program flux_difference'
c
c     Folders
      folder1='./MLS1a_cont'
      folder2='./MLS2b_cont'
      call add_slash_if_absent(folder1)
      call add_slash_if_absent(folder2)
c
c     FLuxes @ ground
      file_in1=folder1(1:strlen(folder1))//'ground_flux_net_signal.txt'
      file_in2=folder2(1:strlen(folder2))//'ground_flux_net_signal.txt'
      file_out='./dfground_signal.txt'
      call get_difference(file_in1,file_in2,file_out,integral)
      write(*,*) 'Ground radiative forcing:',integral,' W/m^2'
c
c     FLuxes @ 200hPa
      file_in1=folder1(1:strlen(folder1))//'probe_flux_net_signal.txt'
      file_in2=folder2(1:strlen(folder2))//'probe_flux_net_signal.txt'
      file_out='./dfprobe_signal.txt'
      call get_difference(file_in1,file_in2,file_out,integral)
      write(*,*) '200 hPa radiative forcing:',integral,' W/m^2'
c
c     FLuxes @ TOA
      file_in1=folder1(1:strlen(folder1))//'ftoa_signal.txt'
      file_in2=folder2(1:strlen(folder2))//'ftoa_signal.txt'
      file_out='./dftoa_signal.txt'
      call get_difference(file_in1,file_in2,file_out,integral)
      write(*,*) 'TOA radiative forcing:',integral,' W/m^2'
c      
      end



      subroutine get_difference(file_in1,file_in2,file_out,integral)
      implicit none
      include 'max.inc'
      include 'formats.inc'
c     
c     Purpose: perform the whole package (spectral signal of flux difference
c     and spectral integration) for 2 given input files
c     
c     Input:
c       - file_in1: first input file
c       - file_in2: second input file
c       - file_out: output file to generate
c     
c     Output:
c       - integral: integral of the difference
c     
c     I/O
      character*(Nchar_mx) file_in1
      character*(Nchar_mx) file_in2
      character*(Nchar_mx) file_out
      double precision integral
c     temp
      integer i,Nlines1,Nlines2
      character*(Nchar_mx) nlines_file
      integer Nk1
      double precision nu1(1:Nk_mx)
      double precision flux1(1:Nk_mx)
      integer Nk2
      double precision nu2(1:Nk_mx)
      double precision flux2(1:Nk_mx)
      double precision dflux(1:Nk_mx)
c     label
      integer strlen
      character*(Nchar_mx) label
      label='subroutine get_difference'

      nlines_file='./nlines'
      call get_nlines(file_in1,nlines_file,Nlines1)
      call get_nlines(file_in2,nlines_file,Nlines2)
      if (Nlines1.ne.Nlines2) then
         call error(label)
         write(*,*) 'Nlines1=',Nlines1
         write(*,*) 'Nlines2=',Nlines2
         stop
      endif

      call read_flux(file_in1,Nlines1,Nk1,nu1,flux1)
      call read_flux(file_in2,Nlines2,Nk2,nu2,flux2)
      do i=1,Nk1
         dflux(i)=flux2(i)-flux1(i)
      enddo                     ! i

      write(*,*) 'Recording difference spectra...'
      open(11,file=file_out(1:strlen(file_out)))
      write(11,10) 'nu (cm^-1) / dF+(TOA) (W/m^2/cm^-1)'
      do i=1,Nk1
         write(11,52) nu1(i),dflux(i)
      enddo ! i
      close(11)
      write(*,*) 'Files was generated:'
      write(*,*) file_out(1:strlen(file_out))
      
      write(*,*) 'Spectral integration...'
      call spectral_integration(Nk1,nu1,dflux,integral)

      return
      end
      


      subroutine spectral_integration(Nk,nu,flux,integral)
      implicit none
      include 'max.inc'
      include 'formats.inc'
c     
c     Purpose: to read a TOA flux profile
c     
c     Input:
c       + Nk: number of (nu/flux) values
c       + nu: nu array [cm^-1]
c       + flux: TOA flux [W/m^2/cm^-1]
c     
c     Output:
c       + integral: spectral integral of the input flux
c     
c     I/O
      integer Nk
      double precision nu(1:Nk_mx)
      double precision flux(1:Nk_mx)
      double precision integral
c     temp
      integer i,ios
      double precision sum
c     label
      integer strlen
      character*(Nchar_mx) label
      label='subroutine spectral_integration'

      sum=0.0D+0
      do i=2,Nk-1
         sum=sum+(flux(i)+flux(i-1))*(nu(i)-nu(i-1))
      enddo                     ! i
      integral=sum/2.0D+0

      return
      end

      

      subroutine read_flux(file_in,Nlines,Nk,nu,flux)
      implicit none
      include 'max.inc'
      include 'formats.inc'
c     
c     Purpose: to read a TOA flux profile
c     
c     Input:
c       + file_in: name of the file to read
c       + Nlines: number of lines in the file
c     
c     Output:
c       + Nk: number of (nu/flux) values
c       + nu: nu array [cm^-1]
c       + flux: TOA flux [W/m^2/cm^-1]
c     
c     I/O
      character*(Nchar_mx) file_in
      integer Nlines
      integer Nk
      double precision nu(1:Nk_mx)
      double precision flux(1:Nk_mx)
c     temp
      integer i,ios
c     label
      integer strlen
      character*(Nchar_mx) label
      label='subroutine read_flux'

      open(10,file=file_in(1:strlen(file_in))
     &     ,status='old',iostat=ios)
      if (ios.ne.0) then        ! file not found
         call error(label)
         write(*,*) 'File not found:'
         write(*,*) file_in(1:strlen(file_in))
         stop
      else
         write(*,*) 'Reading file:'
         write(*,*) file_in(1:strlen(file_in))
         Nk=Nlines-1
         if (Nk.gt.Nk_mx) then
            call error(label)
            write(*,*) 'Nk=',Nk
            write(*,*) 'while Nk_mx=',Nk_mx
            stop
         endif
         read(10,*)
         do i=1,Nk
            read(10,52) nu(i),flux(i)
         enddo ! i
      endif
      close(10)
      
      return
      end
      
