      program forcing_profile_1D
      implicit none
      include 'max.inc'
c     
c     Purpose: to compute the radiative forcing at all levels
c     
c     Variables
      
c     temp
      character*(Nchar_mx) folder,file,header
      character*(Nchar_mx) file_in,file_out
      double precision integral
      integer Nlev,Nlev1a,Nlev2b,Nlev4a
      double precision nf_1a(1:Nlev_mx,1:2)
      double precision nf_2b(1:Nlev_mx,1:2)
      double precision nf_4a(1:Nlev_mx,1:2)
      double precision pp_1a,pa_1a,pFup_1a,pFd_1an,pFn_1a
      double precision pp_2b,pa_2b,pFup_2b,pFd_2bn,pFn_2b
      double precision pp_4a,pa_4a,pFup_4a,pFd_4an,pFn_4a
      double precision pp
      double precision rf_2b1a(1:Nlev_mx,1:2)
      double precision rf_4a2b(1:Nlev_mx,1:2)
      double precision prf_2b1a
      double precision prf_4a2b
      double precision rfmin,rfmax
      integer i0,ilev,j,ios
      logical i0_found
      double precision mu,pi,coeff
c     label
      integer strlen
      character*(Nchar_mx) label
      label='program forcing_profile_1D'

      pi=4.0D+0*atan(1.0D+0)

c     Reading net fluxes and total probe fluxes
      folder='/Volumes/USB2-1.5To/Spectral_data/atmospheres/'
     &     //'RTMIP65/RT_results/1D/MLS1a'
      call add_slash_if_absent(folder)
      file='flux_net.txt'
      file_in=folder(1:strlen(folder))
     &     //file(1:strlen(file))
      call read_flux_profile(file_in,Nlev1a,nf_1a)

c     get cos(theta)
      file='cos_theta'
      file_in=folder(1:strlen(folder))
     &     //file(1:strlen(file))
      open(11,file=file_in(1:strlen(file_in))
     &     ,status='old',iostat=ios)
      if (ios.ne.0) then        ! file not found
         call error(label)
         write(*,*) 'File not found:'
         write(*,*) file_in(1:strlen(file_in))
         stop
      else
         read(11,*) mu
      endif
      close(11)
      coeff=pi/mu
c     
      file='total_probe_flux.txt'
      file_in=folder(1:strlen(folder))
     &     //file(1:strlen(file))
      call read_total_probe_flux(file_in,pp_1a,pa_1a,
     &     pFup_1a,pFd_1an,pFn_1a)

c     -----------------------------------------------------------------------
      write(*,*) '1a results:'
      write(*,*) 'Fnet(ground)=',nf_1a(1,2)*coeff
      write(*,*) 'Fnet(200hPa)=',pFn_1a*coeff
      write(*,*) 'Fnet(TOM)=',nf_1a(Nlev1a,2)*coeff
c     -----------------------------------------------------------------------
      
      folder='/Volumes/USB2-1.5To/Spectral_data/atmospheres/'
     &     //'RTMIP65/RT_results/1D/MLS2b'
      call add_slash_if_absent(folder)
      file='flux_net.txt'
      file_in=folder(1:strlen(folder))
     &     //file(1:strlen(file))
      call read_flux_profile(file_in,Nlev2b,nf_2b)
c     
      file='total_probe_flux.txt'
      file_in=folder(1:strlen(folder))
     &     //file(1:strlen(file))
      call read_total_probe_flux(file_in,pp_2b,pa_2b,
     &     pFup_2b,pFd_2bn,pFn_2b)
      
c     -----------------------------------------------------------------------
      write(*,*) '2b results:'
      write(*,*) 'Fnet(ground)=',nf_2b(1,2)*coeff
      write(*,*) 'Fnet(200hPa)=',pFn_2b*coeff
      write(*,*) 'Fnet(TOM)=',nf_2b(Nlev2b,2)*coeff
c     -----------------------------------------------------------------------
      
      folder='/Volumes/USB2-1.5To/Spectral_data/atmospheres/'
     &     //'RTMIP65/RT_results/1D/MLS4a'
      call add_slash_if_absent(folder)
      file='flux_net.txt'
      file_in=folder(1:strlen(folder))
     &     //file(1:strlen(file))
      call read_flux_profile(file_in,Nlev4a,nf_4a)
c     
      file='total_probe_flux.txt'
      file_in=folder(1:strlen(folder))
     &     //file(1:strlen(file))
      call read_total_probe_flux(file_in,pp_4a,pa_4a,
     &     pFup_4a,pFd_4an,pFn_4a)
      
c     -----------------------------------------------------------------------
      write(*,*) '4a results:'
      write(*,*) 'Fnet(ground)=',nf_4a(1,2)*coeff
      write(*,*) 'Fnet(200hPa)=',pFn_4a*coeff
      write(*,*) 'Fnet(TOM)=',nf_4a(Nlev4a,2)*coeff
c     -----------------------------------------------------------------------
      

c     Computing radiative forcings, in W/m^2/cm^-1
      write(*,*) 'Computing forcings...'
      call radiative_forcing_profile(Nlev2b,Nlev1a,nf_2b,nf_1a,rf_2b1a)
      call radiative_forcing_profile(Nlev4a,Nlev2b,nf_4a,nf_2b,rf_4a2b)
      prf_2b1a=pFn_1a-pFn_2b
      prf_4a2b=pFn_2b-pFn_4a

c     Adding probe radiative forcings to the "rf_*" arrays
      i0_found=.false.
      Nlev=Nlev1a
      pp=pp_1a
      do ilev=1,Nlev-1
         if ((nf_1a(ilev,1).gt.pp).and.(nf_1a(ilev+1,1).lt.pp)) then
            i0_found=.true.
            i0=ilev
            goto 111
         endif
      enddo                     ! ilev
 111  continue
      if (.not.i0_found) then
         call error(label)
         write(*,*) 'i0_found=',i0_found
         write(*,*) 'pp=',pp,' atm'
         do ilev=1,Nlev
            write(*,*) 'p(',ilev,')=',nf_1a(ilev,1),' atm'
         enddo                  ! ilev
         stop
      else
         if (Nlev+1.gt.Nlev_mx) then
            call error(label)
            write(*,*) 'Nlev+1=',Nlev+1
            stop
         endif
         do ilev=Nlev+1,i0+2,-1
            do j=1,2
               rf_2b1a(ilev,j)=rf_2b1a(ilev-1,j)
               rf_4a2b(ilev,j)=rf_4a2b(ilev-1,j)
            enddo               ! j
         enddo                  ! ilev
         rf_2b1a(i0+1,1)=pp
         rf_2b1a(i0+1,2)=prf_2b1a
         rf_4a2b(i0+1,1)=pp
         rf_4a2b(i0+1,2)=prf_4a2b
         Nlev=Nlev+1
      endif
      
c     convert to "equivalent 3D results"
      do ilev=1,Nlev
         rf_2b1a(ilev,2)=rf_2b1a(ilev,2)*coeff
         rf_4a2b(ilev,2)=rf_4a2b(ilev,2)*coeff
      enddo                     ! ilev
c     -----------------------------------------------------------------------
      write(*,*) 'Radiative forcings:'
      write(*,*) '2b-1a'
      write(*,*) 'ground:',rf_2b1a(1,2)
      write(*,*) '200hPa:',prf_2b1a*coeff
      write(*,*) 'TOM:',rf_2b1a(Nlev,2)
      write(*,*) '4a-2b'
      write(*,*) 'ground:',rf_4a2b(1,2)
      write(*,*) '200hPa:',prf_4a2b*coeff
      write(*,*) 'TOM:',rf_4a2b(Nlev,2)
c     -----------------------------------------------------------------------

c     looking for min and max radiative forcings
      rfmax=rf_2b1a(1,2)
      do ilev=1,Nlev
         if (rf_2b1a(ilev,2).gt.rfmax) then
            rfmax=rf_2b1a(ilev,2)
         endif
         if (rf_4a2b(ilev,2).gt.rfmax) then
            rfmax=rf_4a2b(ilev,2)
         endif
      enddo                     ! ilev
      rfmin=rfmax
      do ilev=1,Nlev
         if (rf_2b1a(ilev,2).lt.rfmin) then
            rfmin=rf_2b1a(ilev,2)
         endif
         if (rf_4a2b(ilev,2).lt.rfmin) then
            rfmin=rf_4a2b(ilev,2)
         endif
      enddo                     ! ilev

c     Records
      write(*,*) 'Recording results...'
      
      file_out='./rad_forcing_profile_1D_2b-1a.txt'
      header='pressure [atm] / '
     &     //'2b-1a forcing [W/m^2]'
      call record_flux_profile(file_out,header,Nlev,rf_2b1a)
      write(*,*) 'File was created:'
      write(*,*) file_out(1:strlen(file_out))
      
      file_out='./rad_forcing_profile_1D_4a-2b.txt'
      header='pressure [atm] / '
     &     //'4a-2b forcing [W/m^2]'
      call record_flux_profile(file_out,header,Nlev,rf_4a2b)
      write(*,*) 'File was created:'
      write(*,*) file_out(1:strlen(file_out))

      file_out='./horizontal_bar_1D.txt'
      open(11,file=file_out(1:strlen(file_out)))
      write(11,*) pp,rfmin
      write(11,*) pp,rfmax
      close(11)

 666  continue

      end
      
