c     Copyright (C)
c     2014-2018 Centre National de la Recherche Scientifique
c     2014-2018 Mesotar
c     
c     This file must be used under the terms of the CeCILL license.
c     This source file is licensed as described in the file COPYING, which
c     you should have received as part of this distribution.  The terms
c     are also available at
c     http://www.cecill.info/licences/Licence_CeCILL_V2.1-en.txt
c
      subroutine H2O_cia_continuum(P,T,x,nu,k_cia_h2o)
      implicit none
      include 'max.inc'
      include 'constants.inc'
c
c     Purpose: to compute the water CIA continuum
c
c     Source:
c     The continuum absorption coefficients are calculated from the sum of
c     empirical self and foreign broadening components as described in Liou,
c     Radiation and Cloud Processes in the Atmosphere, 1992, pp. 36-38.
c
c     Input:
c       + P: pressure (Pa)
c       + T: temperature (K)
c       + x: molar fraction of H2O within the gas mixture
c       + nu: wavenumber (inv. cm)
c
c     Output:
c       + k_cia_h2o: water CIA continuum (m^-1)
c
c     I/O
      double precision P
      double precision T
      double precision x
      double precision nu
      double precision k_cia_h2o
c     temp
      double precision sigma_sTr
      double precision sigma_s
      double precision sigma_h2o
      double precision nmol
      double precision rho,rho_h2o
c     data
      double precision a_h2o,b_h2o,c_h2o,beta_h2o
      double precision rsigma_h2o,Tr_h2o
c     label
      integer strlen
      character*(Nchar_mx) label
      label='subroutine H2O_cia_continuum'

      a_h2o=1.0D-2
      b_h2o=1.2D+2
      c_h2o=6.08D+0
      beta_h2o=2.0D-3
      rsigma_h2o=2.0D-3
      Tr_h2o=2.96D+2

c     [P]=Pa
      nmol=P/(Rgp*T)              ! moles/m^3
c     [M_H2O]=g/mol
      rho_h2o=nmol*x*M_H2O*1.0D-3 ! kg(H2O)/m^3
c     1 g/cm^3 = 1000 kg/m^3
      rho_h2o=rho_h2o*1.0D-3      ! g/cm^3

      sigma_sTr=(a_h2o+b_h2o*dexp(-beta_h2o*nu))
      sigma_s=sigma_sTr*dexp(c_h2o*(Tr_h2o/T-1.0D+0))        ! cm^2.g^-1.atm^-1
      sigma_h2o=sigma_s*P/1.013D+5*(x+rsigma_h2o*(1.0D+0-x)) ! cm^2.g^-1

      k_cia_h2o=1.0D+2*sigma_h2o*rho_h2o ! m^-1
c     Debug
c      write(*,*) P,T,x,nmol,rho_h2o,sigma_sTr,sigma_s,
c     &     sigma_h2o,k_cia_h2o
c      stop
c     Debug

      return
      end
