c     Copyright (C)
c     2014-2018 Centre National de la Recherche Scientifique
c     2014-2018 Mesotar
c     
c     This file must be used under the terms of the CeCILL license.
c     This source file is licensed as described in the file COPYING, which
c     you should have received as part of this distribution.  The terms
c     are also available at
c     http://www.cecill.info/licences/Licence_CeCILL_V2.1-en.txt
c     
      subroutine amagat(P,T,amg)
      implicit none
      include 'max.inc'
c
c     Purpose: to compute the local amagat number
c
c     ---------------------------------------------------------------------------------------------------------------------------
c     From Wikipedia (https://en.wikipedia.org/wiki/Loschmidt_constant):
c     
c     The Loschmidt constant or Loschmidt's number (symbol: n0) is the number of particles (atoms or molecules)
c     of an ideal gas in a given volume (the number density). It is usually quoted at standard temperature and pressure,
c     the 2010 CODATA recommended value [1] is 2.686 7805(24)1025 per cubic metre at 0 C and 1 atm and the 2006 CODATA
c     recommended value [2] was 2.686 7774(47)1025 per cubic metre at 0 C and 1 atm. It is named after the Austrian physicist
c     Johann Josef Loschmidt, who was the first to estimate the physical size of molecules in 1865.[3]
c     The term "Loschmidt constant" is also sometimes used to refer to the Avogadro constant, particularly in German texts.
c
c     The Loschmidt constant is given by the relationship:
c     
c     n0=p0/(kB*T0)
c     where p0 is the pressure, kB is the Boltzmann constant and T0 is the thermodynamic temperature.
c     It is related to the Avogadro constant, NA, by:
c     
c     n0=(p0*NA)/(R*T0)
c     where R is the gas constant.
c
c     Being a measure of number density, the Loschmidt constant is used to define the amagat,
c     a practical unit of number density for gases and other substances:
c
c     1 amagat = n0 = 2.686 7805x10^25 m^-3
c     such that the Loschmidt constant is exactly 1 amagat.
c     ---------------------------------------------------------------------------------------------------------------------------
c
c     Input:
c       + P: pressure (atm)
c       + T: temperature (K)
c
c     Output:
c       + amg: number of amagats
c
c     I/O
      double precision P
      double precision T
      double precision amg
c     temp
c     label
      integer strlen
      character*(Nchar_mx) label
      label='subroutine amagat'

      amg=(P*273.15D+0)/T

      return
      end
