c     Copyright (C)
c     2014-2018 Centre National de la Recherche Scientifique
c     2014-2018 Mesotar
c     
c     This file must be used under the terms of the CeCILL license.
c     This source file is licensed as described in the file COPYING, which
c     you should have received as part of this distribution.  The terms
c     are also available at
c     http://www.cecill.info/licences/Licence_CeCILL_V2.1-en.txt
c
      subroutine base_continuum(Nlev,base_k)
      implicit none
      include 'max.inc'
c     
c     Purpose: to read base continuum generated by kspectrum
c     if the corresponding data file is found
c     
c     Input
c       + Nlev: number of atmospheric levels
c     
c     Output
c       + base_k: base continuum (inverse meters) for each atmospheric level
c     
c     I/O
      integer Nlev
      double precision base_k(1:Nlev_mx)
c     temp
      character*(Nchar_mx) base_filename
      character*(Nchar_mx) nlines_file
      integer nl,Nlev_in_file
      logical file_exist
      integer ilev,itmp
      double precision dtmp
c     label
      integer strlen
      character*(Nchar_mx) label
      label='subroutine base_continuum'

      base_filename='./data/hires_spectra/base_continuum.txt'
      nlines_file='./nlines'
      call get_nlines(base_filename,nlines_file,nl)
      Nlev_in_file=nl-1
      if ((Nlev_in_file.ne.0).and.(Nlev_in_file.ne.Nlev)) then
         call error(label)
         write(*,*) 'Number of atmospheric levels:',Nlev
         write(*,*) 'Number of levels for base continnum=',
     &        Nlev_in_file
         write(*,*) 'should be equal'
         stop
      endif

      inquire(file=base_filename(1:strlen(base_filename)),
     &     exist=file_exist)
      if (file_exist) then
         open(15,file=base_filename(1:strlen(base_filename)))
         read(15,*)
         do ilev=1,Nlev
            read(15,*) itmp,dtmp,base_k(ilev)
         enddo                  ! ilev
         close(15)
         write(*,*) 'Base continuum was found in file:'
         write(*,*) base_filename(1:strlen(base_filename))
      else
         do ilev=1,Nlev
            base_k(ilev)=0.0D+0
         enddo                  ! ilev
         write(*,*) 'WARNING: base continuum file was not found'
      endif

      return
      end
      
