c     Copyright (C)
c     2014-2018 Centre National de la Recherche Scientifique
c     2014-2018 Mesotar
c     
c     This file must be used under the terms of the CeCILL license.
c     This source file is licensed as described in the file COPYING, which
c     you should have received as part of this distribution.  The terms
c     are also available at
c     http://www.cecill.info/licences/Licence_CeCILL_V2.1-en.txt
c
      subroutine cia_conversion(sigma,P,T,kCIA)
      implicit none
      include 'max.inc'
      include 'constants.inc'
c
c     Purpose: convert the HITRAN CIA cross-section (cm^5.molecule^-2) into
c     a absorption coefficient (m^-1)
c
c     Input:
c       + sigma: HITRAN CIA cross-section (cm^5.molecule^-2) interpolated from CIA tables
c       + P: pressure (atm)
c       + T: temperature (K)
c
c     Output:
c       + kCIA: CIA absorption coefficient (m^-1)
c
c     I/O
      double precision sigma
      double precision P
      double precision T
      double precision kCIA
c     temp
      double precision N0
      double precision sigma1
      double precision amg
c     parameters
      double precision P0,T0
      parameter(P0=101325.0D+0) ! Pa
      parameter(T0=273.15D+0) ! K
c     label
      integer strlen
      character*(Nchar_mx) label
      label='subroutine cia_conversion'

      N0=P0/(kBz*T0)*1.0D-6 ! molecules*cm^-3*amg^-1
c     Debug
c      write(*,*) 'N0=',N0
c     Debug
      sigma1=sigma*N0**2.0D+0 ! cm^-1.amagat^-2
      call amagat(P,T,amg) ! [amg]=amagat
      kCIA=1.0D+2*sigma1*amg**2.0D+0 ! m^-1

      return
      end
