c     Copyright (C)
c     2014-2018 Centre National de la Recherche Scientifique
c     2014-2018 Mesotar
c     
c     This file must be used under the terms of the CeCILL license.
c     This source file is licensed as described in the file COPYING, which
c     you should have received as part of this distribution.  The terms
c     are also available at
c     http://www.cecill.info/licences/Licence_CeCILL_V2.1-en.txt
c
      subroutine glass_model(Nlay,alt,temp,Tg,Ts,nu,omega,
     &     emission_altitude,Be,Te,Tmoy)
      implicit none
      include 'max.inc'
c     
c     Purpose: compute various physical quantities that result from the
c     one-glass model of the atmosphere
c     
c     Input:
c       + Nlay: number of atmospheric layers
c       + alt: altitude at each level (m)
c       + temp: temperature profile [K]
c       + Tg: ground temperature [K]
c       + Ts: space temperature [K]
c       + nu: wavenumber [cm^-1]
c       + omega: fraction of the gas-space NER due to each gas layer
c     
c     Output:
c       + emission_altitude: mean emission altitude (integral of z*cf) [m]
c       + Be: average of B*omega [W/m^2/sr/cm^-1]
c       + Te: brightness temperature for Be [K]
c       + Tmoy: average of T*omega [K]
c     
c     I/O
      integer Nlay
      double precision alt(1:Nlev_mx)
      double precision temp(1:Nlev_mx)
      double precision Tg,Ts
      double precision nu
      double precision omega(1:Nlay_mx)
      double precision emission_altitude
      double precision Be,Te,Tmoy
c     temp
      double precision T(0:Nlay_mx)
      double precision Bg,Bs,B(0:Nlay_mx)
      integer i
      logical solution_found
c     functions
      double precision Bnu
c     label
      integer strlen
      character*(Nchar_mx) label
      label='subroutine glass_model'

c     temperature profile
      call mean_Tprofile(Nlay,temp,T)
      call mean_Bprofile(Nlay,Tg,T,Ts,nu,Bg,B,Bs)
c      
c     Computation of emission altitude and emission intensity
      emission_altitude=0.0D+0
      Be=0.0D+0
      Tmoy=0.0D+0
      do i=1,Nlay
         emission_altitude=emission_altitude
     &        +(alt(i)+alt(i+1))*omega(i)/2.0D+0 ! m
         Be=Be+B(i)*omega(i)    ! W/m^2/sr/cm^-1
         Tmoy=Tmoy+T(i)*omega(i) ! K
c     Debug
c         write(*,*) B(i),omega(i),Be
c     Debug
      enddo                     ! i
c     Computation of emission brightness temperature
      call solve_Tb_wavnb(nu,Be,solution_found,Te)
      if (.not.solution_found) then
         Te=-1.0D+0
      endif
      
      return
      end
      
