c     Copyright (C)
c     2014-2018 Centre National de la Recherche Scientifique
c     2014-2018 Mesotar
c     
c     This file must be used under the terms of the CeCILL license.
c     This source file is licensed as described in the file COPYING, which
c     you should have received as part of this distribution.  The terms
c     are also available at
c     http://www.cecill.info/licences/Licence_CeCILL_V2.1-en.txt
c
      subroutine interpolation(Np,x,f,x0,f0)
      implicit none
      include 'max.inc'
c
c     Testing routine, sits on top of "spline interpolation"
c
c     Inputs:
c       + Np: number of x/f(x) values
c       + x: array of "x" values
c       + f: array of "f(x)" values
c       + x0: value of x for which function f has to be interpolated
c
c     Outputs
c       + f0: value of f(x0)
c

c     I/O
      integer Np
      double precision x(1:Np_mx)
      double precision f(1:Np_mx)
      double precision x0,f0
c     temp
      integer Nacc
      double precision xacc(1:Nacc_mx,1:2)
      integer index_acc(1:Nacc_mx,1:2)
      double precision z(1:Np_mx)
      integer valid
c     Debug
      integer i
c     Debug
c     label
      integer strlen
      character*(Nchar_mx) label
      label='subroutine interpolation'

      if (Np.ge.4) then
c     cubic spline interpolation when enough reference points are available
         call acceleration(Np,x,Nacc,xacc,index_acc)
         call cubic_spline(Np,x,f,z,valid)
         call spline_interpolation(Np,x,f,z,
     &        Nacc,xacc,index_acc,x0,f0)
      else if (Np.ge.2) then
c     linear interpolation if only two or three points
         if (Np.eq.2) then
            call linear_interpolation(x(1),x(2),f(1),f(2),x0,f0)
         else if (Np.eq.3) then
            if (x0.le.x(2)) then
               call linear_interpolation(x(1),x(2),f(1),f(2),x0,f0)
            else if (x0.gt.x(2)) then
               call linear_interpolation(x(2),x(3),f(2),f(3),x0,f0)
            else
               call error(label)
               write(*,*) 'could not determine which one is bigger:'
               write(*,*) 'x0=',x0
               write(*,*) 'x(2)=',x(2)
               stop
            endif
         else
            call error(label)
            write(*,*) 'Np was found < 4 and >= 2'
            write(*,*) 'however Np is different from 2 and 3'
            write(*,*) 'Np=',Np
            stop
         endif ! Np=2 or 3
      else if (Np.eq.1) then
c     linear interpolation between zero and the only available point
         call linear_interpolation(0.0D+0,x(1),0.0D+0,f(1),x0,f0)
      else
         call error(label)
         write(*,*) 'Interpolation is impossible with:'
         write(*,*) 'Np=',Np
         stop
      endif

c     Debug
c      write(*,*) label(1:strlen(label)),': in'
c      write(*,*) 'Np=',Np
c      do i=1,Np
c         write(*,*) 'x(',i,')=',x(i)
c      enddo
c      do i=1,Np
c         write(*,*) 'f(',i,')=',f(i)
c      enddo
c      write(*,*) 'x0=',x0
c      write(*,*) 'f0=',f0
c      write(*,*) label(1:strlen(label)),': out'
c     Debug

      return
      end



      subroutine integration(Np,x,f,a,b,A0)
      implicit none
      include 'max.inc'
c
c     Purpose: to perform an integration of function "f" between "a" and "b"
c     given Np couples of [x(i),f(x(i))] values
c
c     Inputs:
c       + Np: number of x/f(x) values
c       + x: array of "x" values
c       + f: array of "f(x)" values
c       + a: lower integration boundary
c       + b: upper integration boundary
c
c     Outputs
c       + A0: value of int(f(x),x=a,b)
c

c     I/O
      integer Np
      double precision x(1:Np_mx)
      double precision f(1:Np_mx)
      double precision a,b,A0
c     temp
      integer Nacc
      double precision xacc(1:Nacc_mx,1:2)
      integer index_acc(1:Nacc_mx,1:2)
      double precision z(1:Np_mx)
      double precision int
      integer valid
c     Debug
      integer i,j
c     Debug
      integer strlen
      character*(Nchar_mx) label
      label='subroutine interpolation'

c     Inconsistencies
      do i=2,Np
         if (x(i).lt.x(i-1)) then
            call error(label)
            write(*,*) 'x(',i,')=',x(i)
            write(*,*) 'is lower than x(',i-1,')=',x(i-1)
            write(*,*) 'x and f arrays must be sorted'
            write(*,*) 'by ascending values of x'
            stop
         endif
      enddo ! i
c     Inconsistencies

      if (Np.ge.4) then
c     cubic spline interpolation when enough reference points are available
         call acceleration(Np,x,Nacc,xacc,index_acc)
c     Debug
c         write(*,*) 'Nacc=',Nacc
c         do i=1,Nacc
c            write(*,*) 'xacc=',(xacc(i,j),j=1,2)
c         enddo
cc         stop
c     Debug
         call cubic_spline(Np,x,f,z,valid)
c     Debug
c         write(*,*) 'cubic_spline OK'
c     Debug
c     Debug
c         write(*,*) 'coucou'
c     Debug
         call spline_integration(Np,x,f,z,Nacc,xacc,index_acc,
     &        a,b,A0)
c     Debug
c         if (A0.lt.0.0D+0) then
c            call error(label)
c            write(*,*) 'A0=',A0
c            write(*,*) 'a=',a,' b=',b
c            write(*,*) 'i / x(i) / f(i)'
c            do i=1,Np
c               write(*,*) i,x(i),f(i)
c            enddo
c            stop
c         endif
c     Debug
      else if (Np.ge.2) then
c     linear interpolation if only two or three points
         if (Np.eq.2) then
            call linear_integration(x(1),x(2),f(1),f(2),a,b,A0)
         else if (Np.eq.3) then
            if ((a.le.x(2)).and.(b.le.x(2))) then
               call linear_integration(x(1),x(2),f(1),f(2),a,b,A0)
            else if ((a.ge.x(2)).and.(b.ge.x(2))) then
               call linear_integration(x(2),x(3),f(2),f(3),a,b,A0)
            else if ((a.le.x(2)).and.(b.ge.x(2))) then
               A0=0.0D+0
               call linear_integration(x(1),x(2),f(1),f(2),a,x(2),int)
               A0=A0+int
               call linear_integration(x(2),x(3),f(2),f(3),x(2),b,int)
               A0=A0+int
            else if ((b.le.x(2)).and.(a.ge.x(2))) then
               A0=0.0D+0
               call linear_integration(x(1),x(2),f(1),f(2),b,x(2),int)
               A0=A0+int
               call linear_integration(x(2),x(3),f(2),f(3),x(2),a,int)
               A0=A0+int
            else
               call error(label)
               write(*,*) 'Incoherent integration limits:'
               write(*,*) 'a=',a
               write(*,*) 'b=',b
               stop
            endif
         else
            call error(label)
            write(*,*) 'Np was found < 4 and >= 2'
            write(*,*) 'however Np is different from 2 and 3'
            write(*,*) 'Np=',Np
            stop
         endif ! Np=2 or 3
      else if (Np.eq.1) then
c     linear integration between zero and the only available point
         call linear_integration(0.0D+0,x(1),0.0D+0,f(1),a,b,A0)
      else
         call error(label)
         write(*,*) 'Integration is impossible with:'
         write(*,*) 'Np=',Np
         stop
      endif
c     Debug
c      if (A0.lt.0.0D+0) then
c         call error(label)
c         write(*,*) 'A0=',A0
c         write(*,*) 'Np=',Np
c         stop
c      endif
c     Debug

      return
      end



      subroutine linear_interpolation(x1,x2,f1,f2,x0,f0)
      implicit none
      include 'max.inc'
c
c     Purpose: to perform a linear interpolation between two reference values
c
c     Inputs:
c       + x1, x2: values of x for the two reference points
c       + f1, f2: f1=f(x1) and f2=f(x2), known values of f at points x1 and x2
c       + x0: value of x where interpolation has to be performed
c     
c     Output:
c       + f0: value of f interpolated at x=x0
c

c     I/O
      double precision x1,x2
      double precision f1,f2
      double precision x0,f0
c     temp
c     label
      integer strlen
      character*(Nchar_mx) label
      label='subroutine linear_interpolation'

      if (x1.eq.x2) then
         call error(label)
         write(*,*) 'x1=',x1
         write(*,*) 'x2=',x2
         write(*,*) 'have identical values'
         stop
      else
         f0=f1+(f2-f1)/(x2-x1)*(x0-x1)
      endif

      return
      end



      subroutine linear_integration(x1,x2,f1,f2,a,b,int)
      implicit none
      include 'max.inc'
c
c     Purpose: to perform an integration assuming
c     linear interpolation between two reference values
c
c     Inputs:
c       + x1, x2: values of x for the two reference points
c       + f1, f2: f1=f(x1) and f2=f(x2), known values of f at points x1 and x2
c       + a,b: integration boundaries
c     
c     Output:
c       + int: value of int(f(x),x=a,b)
c

c     I/O
      double precision x1,x2
      double precision f1,f2
      double precision a,b,int
c     temp
      double precision alpha,beta
      integer strlen
      character*(Nchar_mx) label
      label='subroutine linear_integration'

      if ((x1.eq.x2).and.(f1.ne.f2)) then
         call error(label)
         write(*,*) 'x1=',x1
         write(*,*) 'x2=',x2
         write(*,*) 'have identical values'
         write(*,*) 'while:'
         write(*,*) 'f1=',f1
         write(*,*) 'f2=',f2
         write(*,*) 'are different'
         stop
      else
         alpha=(f2-f1)/(x2-x1)
         beta=f1
         int=alpha/2.0D+0*(b**2.0D+0-a**2.0D+0)
     &        +(beta-alpha*x1)*(b-a)
      endif

      return
      end



      subroutine exponential_interpolation(x1,x2,f1,f2,x0,f0)
      implicit none
      include 'max.inc'
c
c     Purpose: to perform an exponential interpolation between two reference values
c
c     Inputs:
c       + x1, x2: values of x for the two reference points
c       + f1, f2: f1=f(x1) and f2=f(x2), known values of f at points x1 and x2
c                 and we assume f(x)=f(x1)*exp[-alpha*(x-x1)] between f1(x1) and f2(x2)
c       + x0: value of x where interpolation has to be performed
c     
c     Output:
c       + f0: value of f interpolated at x=x0
c

c     I/O
      double precision x1,x2
      double precision f1,f2
      double precision x0,f0
c     temp
      double precision alpha
      integer strlen
      character*(Nchar_mx) label
      label='subroutine exponential_interpolation'

      if ((x1.eq.x2).and.(f1.ne.f2)) then
         call error(label)
         write(*,*) 'x1=',x1
         write(*,*) 'x2=',x2
         write(*,*) 'have identical values'
         write(*,*) 'while:'
         write(*,*) 'f1=',f1
         write(*,*) 'f2=',f2
         write(*,*) 'are different'
         stop
      else
         alpha=-dlog(f1/f2)/(x2-x1)
         f0=f1*dexp(-alpha*(x0-x1))
      endif

      return
      end



      subroutine log_interpolation(x1,x2,f1,f2,x0,f0)
      implicit none
      include 'max.inc'
c
c     Purpose: to perform a log interpolation between two reference values
c
c     Inputs:
c       + x1, x2: values of x for the two reference points
c       + f1, f2: f1=f(x1) and f2=f(x2), known values of f at points x1 and x2
c                 and we assume f(x)=f(x1)+1/alpha*dlog[x1/x] between f1(x1) and f2(x2)
c       + x0: value of x where interpolation has to be performed
c     
c     Output:
c       + f0: value of f interpolated at x=x0
c

c     I/O
      double precision x1,x2
      double precision f1,f2
      double precision x0,f0
c     temp
      double precision alpha
      integer strlen
      character*(Nchar_mx) label
      label='subroutine log_interpolation'

      if ((x1.eq.x2).and.(f1.ne.f2)) then
         call error(label)
         write(*,*) 'x1=',x1
         write(*,*) 'x2=',x2
         write(*,*) 'have identical values'
         write(*,*) 'while:'
         write(*,*) 'f1=',f1
         write(*,*) 'f2=',f2
         write(*,*) 'are different'
         stop
      else
         alpha=-dlog(f1/f2)/(x2-x1)
         if (alpha.le.0.0D+0) then
            call error(label)
            write(*,*) 'alpha=',alpha
            write(*,*) 'with:'
            write(*,*) 'f(',x1,')=',f1
            write(*,*) 'f(',x2,')=',f2
            stop
         else
            f0=f1+1.0D+0/alpha*dlog(x1/x0)
         endif
      endif

      return
      end
