c     Copyright (C)
c     2014-2018 Centre National de la Recherche Scientifique
c     2014-2018 Mesotar
c     
c     This file must be used under the terms of the CeCILL license.
c     This source file is licensed as described in the file COPYING, which
c     you should have received as part of this distribution.  The terms
c     are also available at
c     http://www.cecill.info/licences/Licence_CeCILL_V2.1-en.txt
c
      subroutine num2str(num,str,error)
      implicit none
      include 'max.inc'
      include 'formats.inc'
c
c     Purpose: to convert an integer to a character string
c
c     Input:
c       + num: integer
c     
c     Output:
c       + str: character string
c       + error: a value of zero means conversion is OK; a value of 1 means "num"
c                was not recognized.
c

c     I/O
      integer num
      character*(Nchar_mx) str
      integer error
c     temp
      integer absnum,sign
      character*(Nchar_mx) f
c     label
      integer strlen
      character*(Nchar_mx) label
      label='subroutine num2str'

      if (num.lt.0) then
         sign=-1
      else
         sign=1
      endif
      absnum=abs(num)
      
      error=0
      if ((absnum.ge.0).and.(absnum.lt.10)) then
         write(str,11) absnum
      else if ((absnum.ge.10).and.(absnum.lt.100)) then
         write(str,12) absnum
      else if ((absnum.ge.100).and.(absnum.lt.1000)) then
         write(str,13) absnum
      else if ((absnum.ge.1000).and.(absnum.lt.10000)) then
         write(str,14) absnum
      else if ((absnum.ge.10000).and.(absnum.lt.100000)) then
         write(str,15) absnum
      else if ((absnum.ge.100000).and.(absnum.lt.1000000)) then
         write(str,16) absnum
      else if ((absnum.ge.1000000).and.(absnum.lt.10000000)) then
         write(str,17) absnum
      else if ((absnum.ge.10000000).and.(absnum.lt.100000000)) then
         write(str,18) absnum
      else if ((absnum.ge.100000000).and.(absnum.lt.1000000000)) then
         write(str,19) absnum
      else
         error=1
         goto 666
      endif

      if (sign.eq.-1) then
         str="-"//str(1:strlen(str))
      endif
      
 666  continue
      return
      end



      subroutine num2str3(num,str3)
      implicit none
      include 'max.inc'
c
c     Purpose: to convert an integer to a character string of size 3
c
c     Input:
c       + num: integer
c     
c     Output:
c       + str3: character string
c

c     I/O
      integer num
      character*3 str3
c     temp
      character*(Nchar_mx) str_tmp
      integer err_code
c     label
      integer strlen
      character*(Nchar_mx) label
      label='subroutine num2str3'

      call num2str(num,str_tmp,err_code)
      if (err_code.eq.1) then
         call error(label)
         write(*,*) 'Problem converting to character string'
         write(*,*) 'num=',num
         stop
      endif

      if (num.lt.10) then
         str3='00'//
     &        str_tmp(1:strlen(str_tmp))
      else if (num.lt.100) then
         str3='0'//
     &        str_tmp(1:strlen(str_tmp))
      else if (num.lt.1000) then
         str3=str_tmp(1:strlen(str_tmp))
      else
         call error(label)
         write(*,*) 'unable to convert:'
         write(*,*) 'num=',num,' > 1000'
         write(*,*) 'to a string of 3 characters'
         stop
      endif

      return
      end




      subroutine num2str2(num,str2)
      implicit none
      include 'max.inc'
c
c     Purpose: to convert an integer to a character string of size 2
c
c     Input:
c       + num: integer
c     
c     Output:
c       + str2: character string
c

c     I/O
      integer num
      character*2 str2
c     temp
      character*(Nchar_mx) str_tmp
      integer err_code
c     label
      integer strlen
      character*(Nchar_mx) label
      label='subroutine num2str2'

      call num2str(num,str_tmp,err_code)
      if (err_code.eq.1) then
         call error(label)
         write(*,*) 'Problem converting to character string'
         write(*,*) 'num=',num
         stop
      endif

      if (num.lt.10) then
         str2='0'//
     &        str_tmp(1:strlen(str_tmp))
      else if (num.lt.100) then
         str2=str_tmp(1:strlen(str_tmp))
      else
         call error(label)
         write(*,*) 'unable to convert:'
         write(*,*) 'num=',num,' > 100'
         write(*,*) 'to a string of 2 characters'
         stop
      endif

      return
      end



      subroutine dble2str(num,nap,str,err)
      implicit none
      include 'max.inc'
      include 'formats.inc'
c
c     Purpose: to convert a double precision to a character string
c
c     Input:
c       + num: double precision
c       + nap: number of digits after the "." that are converted to character
c     
c     Output:
c       + str: character string
c       + err: a value of zero means conversion is OK; a value of 1 means "num"
c                was not recognized.
c

c     I/O
      double precision num
      integer nap
      character*(Nchar_mx) str
      integer err
c     temp
      integer sign
      integer i,j,idx,t
      character*1 str1
      character*3 str3
      integer n
      double precision absnum,tmp,a
      character*(Nchar_mx) nstr
      integer strlen
      character*(Nchar_mx) label
      label='subroutine num2str'

      if (num.eq.0.0D+0) then
         str='0.00'
         err=0
         goto 666
      endif

      if (num.lt.0.0D+0) then
         sign=-1
      else
         sign=1
      endif
      absnum=dabs(num)
      
      err=0
      n=0
      tmp=absnum
      if (absnum.gt.1.0D+0) then
         do while (tmp.gt.1.0D+0)
            n=n+1
            tmp=tmp/1.0D+1
c     Debug
c            write(*,*) tmp,n
c     Debug
         enddo
      else if (absnum.lt.1.0D+0) then
         do while (tmp.lt.1.0D+0)
            n=n-1
            tmp=tmp*1.0D+1
         enddo
      else if (absnum.eq.1.0D+0) then
      else
         call error(label)
         write(*,*) 'absnum=',absnum
         stop
      endif
      a=tmp
c     at this point, absnum=a*10^n
c     Debug
c      write(*,*) 'a=',a
c      write(*,*) 'n=',n
c     Debug

      str=''
      idx=int(a)
      write(str1,11) idx
      t=idx*10
      str=str(1:strlen(str))
     &     //str1(1:strlen(str1))
      str=str(1:strlen(str))
     &     //'.'
      do i=1,nap
         idx=int(a*10**i-t)
c     Debug
c         write(*,*) 'i=',i,' idx=',idx
c     Debug
         t=10*(t+idx)
         write(str1,11) idx
         str=str(1:strlen(str))
     &        //str1(1:strlen(str1))
      enddo ! i
c     Debug
c      write(*,*) 'str=',str(1:strlen(str))
c     Debug

      if (n.ne.0) then
         call num2str3(abs(n),str3)
         if (n.lt.0) then
            nstr='-'//str3(1:strlen(str3))
         else
            nstr=str3(1:strlen(str3))
         endif
         str=str(1:strlen(str))
     &        //'E'
     &        //nstr(1:strlen(nstr))
c     &        //'}'
      endif

      if (sign.eq.-1) then
         str="-"//str(1:strlen(str))
      endif
      
 666  continue
      return
      end



      subroutine dble2str_noexp(num,nap,str,err)
      implicit none
      include 'max.inc'
      include 'formats.inc'
c
c     Purpose: to convert a double precision to a character string
c     using a natural "+/-XXXX.XXXXX" convention
c
c     Input:
c       + num: double precision
c       + nap: number of digits after the "." that are converted to character
c     
c     Output:
c       + str: character string
c       + err: a value of zero means conversion is OK; a value of 1 means "num"
c                was not recognized.
c

c     I/O
      double precision num
      integer nap
      character*(Nchar_mx) str
      integer err
c     temp
      integer sign
      integer i,j,idx,t
      character*1 str1,zero_ch
      character*3 str3
      integer n
      double precision absnum,tmp,a
      character*(Nchar_mx) nstr
      integer strlen
      character*(Nchar_mx) label
      label='subroutine dble2str_noexp'

      if (num.eq.0.0D+0) then
         str='0.00'
         err=0
         goto 666
      endif

      if (num.lt.0.0D+0) then
         sign=-1
      else
         sign=1
      endif
      absnum=dabs(num)
      
      err=0
      n=0
      tmp=absnum
      write(zero_ch,11) 0
      if (absnum.gt.1.0D+0) then
         do while (tmp.ge.1.0D+0)
            n=n+1
            tmp=tmp/1.0D+1
c     Debug
c            write(*,*) tmp,n
c     Debug
         enddo
         n=n-1
      else if (absnum.lt.1.0D+0) then
         do while (tmp.lt.1.0D+0)
            n=n-1
            tmp=tmp*1.0D+1
         enddo
      else if (absnum.eq.1.0D+0) then
      else
         call error(label)
         write(*,*) 'absnum=',absnum
         stop
      endif
      a=tmp
c     at this point, absnum=a*10^n
c     Debug
c      write(*,*) 'a=',a
c      write(*,*) 'n=',n
c     Debug

      str=''
      if (n.ge.0) then
         tmp=absnum
         do i=n,0,-1
            idx=tmp/(10**i)
            tmp=tmp-idx*10**i
            write(str1,11) idx
            str=str(1:strlen(str))
     &           //str1(1:strlen(str1))
         enddo                  ! i
         str=str(1:strlen(str))
     &        //'.'
         do i=1,nap
            tmp=tmp*10
            idx=int(tmp)
            tmp=tmp-dble(idx)
            write(str1,11) idx
            str=str(1:strlen(str))
     &           //str1(1:strlen(str1))
         enddo ! i
      else
         str=zero_ch(1:strlen(zero_ch))//'.'
         do i=1,abs(n+1)
            str=str(1:strlen(str))
     &           //zero_ch(1:strlen(zero_ch))
         enddo ! i
         tmp=absnum*10**abs(n+1)
         do i=1,nap
            tmp=tmp*10
            idx=int(tmp)
            tmp=tmp-dble(idx)
            write(str1,11) idx
            str=str(1:strlen(str))
     &           //str1(1:strlen(str1))
         enddo ! i
      endif

      if (sign.eq.-1) then
         str="-"//str(1:strlen(str))
      endif
      
 666  continue
      return
      end
