c     Copyright (C)
c     2014-2018 Centre National de la Recherche Scientifique
c     2014-2018 Mesotar
c     
c     This file must be used under the terms of the CeCILL license.
c     This source file is licensed as described in the file COPYING, which
c     you should have received as part of this distribution.  The terms
c     are also available at
c     http://www.cecill.info/licences/Licence_CeCILL_V2.1-en.txt
c
      subroutine record_Tprofile(Nlay,alt,Tg,T,Ts)
      implicit none
      include 'max.inc'
      include 'formats.inc'
c
c     Purpose: to record the temperature profile
c
c     Input:
c       + Nlay: number of atmospheric layers
c       + alt: altitude levels [m]
c       + Tg: ground temperature [K]
c       + T: mean temperature of each gas layer [K]
c       + Ts: space temperature [K]
c      
c     Output:
c
c     I/O
      integer Nlay
      double precision alt(1:Nlev_mx)
      double precision Tg
      double precision T(0:Nlay_mx)
      double precision Ts
c     temp
      integer i
      character*(Nchar_mx) res_file,line
c     label
      integer strlen
      character*(Nchar_mx) label
      label='subroutine record_Tprofile'

      res_file='./results/Tprofile.txt'
      line='altitude (km) / temperature (K)'
      open(14,file=res_file(1:strlen(res_file)))
      write(14,10) line(1:strlen(line))
      write(14,52) alt(1)/1.0D+3,Tg
      do i=1,Nlay
         write(14,52) alt(i)/1.0D+3,T(i)
      enddo                     ! i
      write(14,52) alt(Nlay+1)/1.0D+3,Ts
      close(14)
      write(*,*) '---'
      write(*,*) 'Output file was generated:'
      write(*,*) res_file(1:strlen(res_file))

      return
      end


      
      subroutine record_ftoa_signal(pass,ai,Nk,nu,ftoa_signal)
      implicit none
      include 'max.inc'
      include 'formats.inc'
c
c     Purpose: to append data to the TOA flux signal output file
c
c     Input:
c       + pass: index of the pass over spectral discretization
c       + ai: true if angular integration is enabled
c       + Nk: number of values of nu
c       + nu: these "Nk" values of nu
c       + ftoa_signal: TOA flux for these "Nk" values (W/m^2/cm^-1)
c
c     Output:
c
c     I/O
      integer pass,Nk
      logical ai
      double precision nu(1:Nk_mx)
      double precision ftoa_signal(1:Nk_mx) ! W/m^2/cm^-1
c     temp
      integer i,nl,ik,ik1
      character*(Nchar_mx) nlines_file
      character*(Nchar_mx) res_file,gra_file,line
c     label
      integer strlen
      character*(Nchar_mx) label
      label='subroutine record_ftoa_signal'

      res_file='./results/ftoa_signal.txt'
      line='nu (cm^-1) / F+(TOA) (W/m^2/cm^-1)'
      nlines_file='./nlines'
c      call get_nlines(res_file,nlines_file,nl)

      open(14,file=res_file(1:strlen(res_file)),access='append')
c      if (nl.gt.0) then
c         do i=1,nl
c            read(14,*)
c         enddo
c      endif
      if (pass.eq.1) then
         ik1=1
         write(14,10) line(1:strlen(line))
      else
         ik1=2
      endif
      do ik=ik1,Nk
         write(14,52) nu(ik),ftoa_signal(ik)
      enddo                     ! ik
      close(14)
      write(*,*) '---'
      if (pass.eq.1) then
         write(*,*) 'Output file was generated:'
      else
         write(*,*) 'Output file was updated:'
      endif
      write(*,*) res_file(1:strlen(res_file))
c     
      gra_file='./results/gra_Ftoa'
      open(10,file=gra_file(1:strlen(gra_file)))
      write(10,10) 'set term pdf enhanced color '
     &     //'font "helvetica,20" linewidth 1 size 10,6'
      write(10,10) 'set output "Ftoa.pdf"'
      write(10,10) 'set xlabel "{/Symbol n} (cm^{-1})"'
      if (ai) then
         write(10,10) 'set ylabel "TOA Flux (W/m^{2}/cm^{-1})"'
      else
         write(10,10) 'set ylabel "TOA Flux (W/m^{2}/sr/cm^{-1})"'
      endif
      write(10,10) 'plot "./ftoa_signal.txt" u 1:2 notitle '
     &     //'w lines lt -1 lw 1'
      write(10,10) 'quit'
      close(10)

      return
      end


      
      subroutine record_pflux_signal(pass,ai,Nk,nu,
     &     pfup_signal,pfdn_signal,pfnet_signal)
      implicit none
      include 'max.inc'
      include 'formats.inc'
c
c     Purpose: to append data to the "probe" flux signals output file
c
c     Input:
c       + pass: index of the pass over spectral discretization
c       + ai: true if angular integration is enabled
c       + Nk: number of values of nu
c       + nu: these "Nk" values of nu
c       + pfup_signal: probe upward flux for these "Nk" values (W/m^2/cm^-1)
c       + pfdn_signal: probe downward flux for these "Nk" values (W/m^2/cm^-1)
c       + pfnet_signal: probe net flux for these "Nk" values (W/m^2/cm^-1)
c
c     Output:
c
c     I/O
      integer pass,Nk
      logical ai
      double precision nu(1:Nk_mx)
      double precision pfup_signal(1:Nk_mx) ! W/m^2/cm^-1
      double precision pfdn_signal(1:Nk_mx) ! W/m^2/cm^-1
      double precision pfnet_signal(1:Nk_mx) ! W/m^2/cm^-1
c     temp
      integer i,nl,ik,ik1
      character*(Nchar_mx) nlines_file
      character*(Nchar_mx) res_file,gra_file,line
c     label
      integer strlen
      character*(Nchar_mx) label
      label='subroutine record_pflux_signal'

      res_file='./results/probe_flux_up_signal.txt'
      line='nu (cm^-1) / F+(probe) (W/m^2/cm^-1)'
      open(14,file=res_file(1:strlen(res_file)),access='append')
      if (pass.eq.1) then
         ik1=1
         write(14,10) line(1:strlen(line))
      else
         ik1=2
      endif
      do ik=ik1,Nk
         write(14,52) nu(ik),pfup_signal(ik)
      enddo                     ! ik
      close(14)
      write(*,*) '---'
      if (pass.eq.1) then
         write(*,*) 'Output file was generated:'
      else
         write(*,*) 'Output file was updated:'
      endif
      write(*,*) res_file(1:strlen(res_file))
      if (pass.eq.1) then
         gra_file='./results/gra_probe_flux_up'
         open(10,file=gra_file(1:strlen(gra_file)))
         write(10,10) 'set xlabel "{/Symbol n} (cm^{-1})"'
         if (ai) then
            write(10,10) 'set ylabel "Upward flux at probe pressure '
     &           //'(W/m^{2}/cm^{-1})"'
         else
            write(10,10) 'set ylabel "Upward flux at probe pressure '
     &           //'(W/m^{2}/sr/cm^{-1})"'
         endif
         write(10,10) 'plot "./probe_flux_up_signal.txt" '
     &        //'u 1:2 notitle w lines lt -1 lw 1'
         write(10,10) 'quit'
         close(10)
      endif
c      
      res_file='./results/probe_flux_dn_signal.txt'
      line='nu (cm^-1) / F-(probe) (W/m^2/cm^-1)'
      open(14,file=res_file(1:strlen(res_file)),access='append')
      if (pass.eq.1) then
         ik1=1
         write(14,10) line(1:strlen(line))
      else
         ik1=2
      endif
      do ik=ik1,Nk
         write(14,52) nu(ik),pfdn_signal(ik)
      enddo                     ! ik
      close(14)
      write(*,*) '---'
      if (pass.eq.1) then
         write(*,*) 'Output file was generated:'
      else
         write(*,*) 'Output file was updated:'
      endif
      write(*,*) res_file(1:strlen(res_file))
      if (pass.eq.1) then
         gra_file='./results/gra_probe_flux_dn'
         open(10,file=gra_file(1:strlen(gra_file)))
         write(10,10) 'set xlabel "{/Symbol n} (cm^{-1})"'
         if (ai) then
            write(10,10) 'set ylabel "Downward flux at probe pressure '
     &           //'(W/m^{2}/cm^{-1})"'
         else
            write(10,10) 'set ylabel "Downward flux at probe pressure '
     &           //'(W/m^{2}/sr/cm^{-1})"'
         endif
         write(10,10) 'plot "./probe_flux_dn_signal.txt" '
     &        //'u 1:2 notitle w lines lt -1 lw 1'
         write(10,10) 'quit'
         close(10)
      endif
c      
      res_file='./results/probe_flux_net_signal.txt'
      line='nu (cm^-1) / Fnet(probe) (W/m^2/cm^-1)'
      open(14,file=res_file(1:strlen(res_file)),access='append')
      if (pass.eq.1) then
         ik1=1
         write(14,10) line(1:strlen(line))
      else
         ik1=2
      endif
      do ik=ik1,Nk
         write(14,52) nu(ik),pfnet_signal(ik)
      enddo                     ! ik
      close(14)
      write(*,*) '---'
      if (pass.eq.1) then
         write(*,*) 'Output file was generated:'
      else
         write(*,*) 'Output file was updated:'
      endif
      write(*,*) res_file(1:strlen(res_file))
      if (pass.eq.1) then
         gra_file='./results/gra_probe_flux_net'
         open(10,file=gra_file(1:strlen(gra_file)))
         write(10,10) 'set xlabel "{/Symbol n} (cm^{-1})"'
         if (ai) then
            write(10,10) 'set ylabel "Net flux at probe pressure '
     &           //'(W/m^{2}/cm^{-1})"'
         else
            write(10,10) 'set ylabel "Net flux at probe pressure '
     &           //'(W/m^{2}/sr/cm^{-1})"'
         endif
         write(10,10) 'plot "./probe_flux_net_signal.txt" '
     &        //'u 1:2 notitle w lines lt -1 lw 1'
         write(10,10) 'quit'
         close(10)
      endif

      return
      end


      
      subroutine record_gflux_signal(pass,ai,Nk,nu,
     &     gfup_signal,gfdn_signal,gfnet_signal)
      implicit none
      include 'max.inc'
      include 'formats.inc'
c
c     Purpose: to append data to the ground flux signals output file
c
c     Input:
c       + pass: index of the pass over spectral discretization
c       + ai: true if angular integration is enabled
c       + Nk: number of values of nu
c       + nu: these "Nk" values of nu
c       + gfup_signal: ground upward flux for these "Nk" values (W/m^2/cm^-1)
c       + gfdn_signal: ground downward flux for these "Nk" values (W/m^2/cm^-1)
c       + gfnet_signal: ground net flux for these "Nk" values (W/m^2/cm^-1)
c
c     Output:
c
c     I/O
      integer pass,Nk
      logical ai
      double precision nu(1:Nk_mx)
      double precision gfup_signal(1:Nk_mx) ! W/m^2/cm^-1
      double precision gfdn_signal(1:Nk_mx) ! W/m^2/cm^-1
      double precision gfnet_signal(1:Nk_mx) ! W/m^2/cm^-1
c     temp
      integer i,nl,ik,ik1
      character*(Nchar_mx) nlines_file
      character*(Nchar_mx) res_file,gra_file,line
c     label
      integer strlen
      character*(Nchar_mx) label
      label='subroutine record_gflux_signal'

      res_file='./results/ground_flux_up_signal.txt'
      line='nu (cm^-1) / F+(probe) (W/m^2/cm^-1)'
      open(14,file=res_file(1:strlen(res_file)),access='append')
      if (pass.eq.1) then
         ik1=1
         write(14,10) line(1:strlen(line))
      else
         ik1=2
      endif
      do ik=ik1,Nk
         write(14,52) nu(ik),gfup_signal(ik)
      enddo                     ! ik
      close(14)
      write(*,*) '---'
      if (pass.eq.1) then
         write(*,*) 'Output file was generated:'
      else
         write(*,*) 'Output file was updated:'
      endif
      write(*,*) res_file(1:strlen(res_file))
c      
      res_file='./results/ground_flux_dn_signal.txt'
      line='nu (cm^-1) / F-(probe) (W/m^2/cm^-1)'
      open(14,file=res_file(1:strlen(res_file)),access='append')
      if (pass.eq.1) then
         ik1=1
         write(14,10) line(1:strlen(line))
      else
         ik1=2
      endif
      do ik=ik1,Nk
         write(14,52) nu(ik),gfdn_signal(ik)
      enddo                     ! ik
      close(14)
      write(*,*) '---'
      if (pass.eq.1) then
         write(*,*) 'Output file was generated:'
      else
         write(*,*) 'Output file was updated:'
      endif
      write(*,*) res_file(1:strlen(res_file))
c      
      res_file='./results/ground_flux_net_signal.txt'
      line='nu (cm^-1) / Fnet(probe) (W/m^2/cm^-1)'
      open(14,file=res_file(1:strlen(res_file)),access='append')
      if (pass.eq.1) then
         ik1=1
         write(14,10) line(1:strlen(line))
      else
         ik1=2
      endif
      do ik=ik1,Nk
         write(14,52) nu(ik),gfnet_signal(ik)
      enddo                     ! ik
      close(14)
      write(*,*) '---'
      if (pass.eq.1) then
         write(*,*) 'Output file was generated:'
      else
         write(*,*) 'Output file was updated:'
      endif
      write(*,*) res_file(1:strlen(res_file))

      return
      end



      subroutine record_cf_signal(pass,Nlev,Nlay,alt,Nk,nu,
     &     cf_signal,omega_signal,emission_altitude,Be,Te,Tmoy)
      implicit none
      include 'max.inc'
      include 'formats.inc'
c
c     Purpose: to append data to the contribution function signal output file
c
c     Input:
c       + pass: index of the pass over spectral discretization
c       + Nlev: number of atmospheric levels
c       + Nlay: number of atmospheric layers
c       + alt: altitude (m) of the atmospheric levels
c       + Nk: number of values of nu
c       + nu: these "Nk" values of nu
c       + cf_signal: contribution function for these "Nk" values
c       + omega_signal: omega function for these "Nk" values
c       + emission_altitude: mean emission altitude (integral of z*cf) [m]
c       + Be: average of B*omega [W/m^2/sr/cm^-1]
c       + Te: brightness temperature for Be [K]
c       + Tmoy: average of T*omega [K]
c
c     Output:
c
c     I/O
      integer pass,Nlev,Nlay
      double precision alt(1:Nlev_mx)
      integer Nk
      double precision nu(1:Nk_mx)
      double precision cf_signal(1:Nk_mx,0:Nlay_mx)
      double precision omega_signal(1:Nk_mx,1:Nlay_mx)
      double precision emission_altitude(1:Nk_mx)
      double precision Be(1:Nk_mx)
      double precision Te(1:Nk_mx)
      double precision Tmoy(1:Nk_mx)
c     temp
      integer i,nl,ik,ik1
      double precision z
      character*(Nchar_mx) nlines_file
      character*(Nchar_mx) res_file,line
c     label
      integer strlen
      character*(Nchar_mx) label
      label='subroutine record_cf_signal'

c     Debug
c      do ik=1,Nk
c         write(*,*) ik,Be(ik),Te(ik)
c      enddo ! ik
c     Debug
      
      res_file='./results/cf_signal.txt'
      line='nu (cm^-1) / Contribution Function'
      nlines_file='./nlines'
      open(14,file=res_file(1:strlen(res_file)),access='append')
      if (pass.eq.1) then
         ik1=1
      else
         ik1=2
      endif
      do ik=ik1,Nk
         do i=0,Nlay
            if (i.eq.0) then
               z=alt(1)/1.0D+3 ! km
            else
               z=(alt(i)+alt(i+1))/2.0D+3 ! km
            endif
            write(14,52) nu(ik),z,cf_signal(ik,i)
         enddo ! i
         write(14,*)
      enddo                     ! ik
      close(14)
      write(*,*) '---'
      if (pass.eq.1) then
         write(*,*) 'Output file was generated:'
      else
         write(*,*) 'Output file was updated:'
      endif
      write(*,*) res_file(1:strlen(res_file))
c
      res_file='./results/omega_function.txt'
      line='nu (cm^-1) / Omega Function'
      nlines_file='./nlines'
      open(14,file=res_file(1:strlen(res_file)),access='append')
      if (pass.eq.1) then
         ik1=1
      else
         ik1=2
      endif
      do ik=ik1,Nk
         do i=1,Nlay
            z=(alt(i)+alt(i+1))/2.0D+3 ! km
            write(14,52) nu(ik),z,omega_signal(ik,i)
         enddo ! i
         write(14,*)
      enddo                     ! ik
      close(14)
      write(*,*) '---'
      if (pass.eq.1) then
         write(*,*) 'Output file was generated:'
      else
         write(*,*) 'Output file was updated:'
      endif
      write(*,*) res_file(1:strlen(res_file))
c
      res_file='./results/emission_altitude.txt'
      line='nu (cm^-1) / Emission Altitude (m)'
      open(14,file=res_file(1:strlen(res_file)),access='append')
      if (pass.eq.1) then
         ik1=1
         write(14,10) line(1:strlen(line))
      else
         ik1=2
      endif
      do ik=ik1,Nk
         write(14,52) nu(ik),emission_altitude(ik)
      enddo                     ! ik
      close(14)
      write(*,*) '---'
      if (pass.eq.1) then
         write(*,*) 'Output file was generated:'
      else
         write(*,*) 'Output file was updated:'
      endif
      write(*,*) res_file(1:strlen(res_file))
c
      res_file='./results/Be.txt'
      line='nu (cm^-1) / Emission radiance (W/m^2/sr/cm^-1)'
      open(14,file=res_file(1:strlen(res_file)),access='append')
      if (pass.eq.1) then
         ik1=1
         write(14,10) line(1:strlen(line))
      else
         ik1=2
      endif
      do ik=ik1,Nk
         write(14,52) nu(ik),Be(ik)
      enddo                     ! ik
      close(14)
      write(*,*) '---'
      if (pass.eq.1) then
         write(*,*) 'Output file was generated:'
      else
         write(*,*) 'Output file was updated:'
      endif
      write(*,*) res_file(1:strlen(res_file))
c
      res_file='./results/Te.txt'
      line='nu (cm^-1) / Emission temperature (K)'
      open(14,file=res_file(1:strlen(res_file)),access='append')
      if (pass.eq.1) then
         ik1=1
         write(14,10) line(1:strlen(line))
      else
         ik1=2
      endif
      do ik=ik1,Nk
         write(14,52) nu(ik),Te(ik)
      enddo                     ! ik
      close(14)
      write(*,*) '---'
      if (pass.eq.1) then
         write(*,*) 'Output file was generated:'
      else
         write(*,*) 'Output file was updated:'
      endif
      write(*,*) res_file(1:strlen(res_file))
c
      res_file='./results/Tmoy.txt'
      line='nu (cm^-1) / Average emission temperature (K)'
      open(14,file=res_file(1:strlen(res_file)),access='append')
      if (pass.eq.1) then
         ik1=1
         write(14,10) line(1:strlen(line))
      else
         ik1=2
      endif
      do ik=ik1,Nk
         write(14,52) nu(ik),Tmoy(ik)
      enddo                     ! ik
      close(14)
      write(*,*) '---'
      if (pass.eq.1) then
         write(*,*) 'Output file was generated:'
      else
         write(*,*) 'Output file was updated:'
      endif
      write(*,*) res_file(1:strlen(res_file))

      return
      end



      subroutine record_Tb_signal(pass,Nk,nu,Tb_signal)
      implicit none
      include 'max.inc'
      include 'formats.inc'
c
c     Purpose: to append data to the brightness temperature signal output file
c
c     Input:
c       + pass: index of the pass over spectral discretization
c       + Nk: number of values of nu
c       + nu: these "Nk" values of nu
c       + Tb_signal: brightness temperature for these "Nk" values (K)
c
c     Output:
c
c     I/O
      integer pass
      integer Nk
      double precision nu(1:Nk_mx)
      double precision Tb_signal(1:Nk_mx)
c     temp
      integer i,nl,ik,ik1
      character*(Nchar_mx) nlines_file
      character*(Nchar_mx) res_file,line
c     label
      integer strlen
      character*(Nchar_mx) label
      label='subroutine record_Tb_signal'

      res_file='./results/Tb_signal.txt'
      line='nu (cm^-1) / Brightness temperature'
      nlines_file='./nlines'
c      call get_nlines(res_file,nlines_file,nl)

      open(14,file=res_file(1:strlen(res_file)),access='append')
c      if (nl.gt.0) then
cc         do i=1,nl
c            read(14,*)
c         enddo
c      endif
      if (pass.eq.1) then
         ik1=1
         write(14,10) line(1:strlen(line))
      else
         ik1=2
      endif
      do ik=ik1,Nk
         write(14,52) nu(ik),Tb_signal(ik) ! cm^-1,K
      enddo                     ! ik
      close(14)
      write(*,*) '---'
      if (pass.eq.1) then
         write(*,*) 'Output file was generated:'
      else
         write(*,*) 'Output file was updated:'
      endif
      write(*,*) res_file(1:strlen(res_file))

      return
      end



      subroutine record_flux_up_signal(pass,Nk,nu,Nlev,alt,pres,
     &     flux_up_signal)
      implicit none
      include 'max.inc'
      include 'formats.inc'
c
c     Purpose: to append data to the upward flux signal output file
c
c     Input:
c       + pass: index of the pass over spectral discretization
c       + Nk: number of values of nu
c       + nu: these "Nk" values of nu
c       + Nlev: number of levels (interfaces)
c       + alt: altitude levels (meters)
c       + pres: pressure levels (atm)
c       + flux_up_signal: upward flux for these "Nk" values
c
c     Output:
c
c     I/O
      integer pass,Nk
      double precision nu(1:Nk_mx)
      integer Nlev
      double precision alt(1:Nlev_mx)
      double precision pres(1:Nlev_mx)
      double precision flux_up_signal(1:Nk_mx,0:Nlay_mx)
c     temp
      integer Nlay,i,nl,ik,ik1
      character*(Nchar_mx) nlines_file
      character*(Nchar_mx) res_file,line
c     label
      integer strlen
      character*(Nchar_mx) label
      label='subroutine record_flux_up_signal'

      Nlay=Nlev-1
      res_file='./results/flux_up_signal.txt'
      line='nu (cm^-1) / F+ (W/m^2)'
      nlines_file='./nlines'
c      call get_nlines(res_file,nlines_file,nl)

      open(14,file=res_file(1:strlen(res_file)),access='append')
c      if (nl.gt.0) then
c         do i=1,nl
c            read(14,*)
c         enddo
c      endif
      if (pass.eq.1) then
         ik1=1
         write(14,10) 'Number of levels:'
         write(14,*) Nlev
         write(14,10) 'Altitudes (meters):'
         write(14,52) (alt(i),i=1,Nlev)
         write(14,10) 'Pressures (atm):'
         write(14,52) (pres(i),i=1,Nlev)
         write(14,*)
         write(14,10) line(1:strlen(line))
      else
         ik1=2
      endif
      do ik=ik1,Nk
         write(14,52) nu(ik),(flux_up_signal(ik,i),i=0,Nlay)
      enddo                     ! ik
      close(14)
      write(*,*) '---'
      if (pass.eq.1) then
         write(*,*) 'Output file was generated:'
      else
         write(*,*) 'Output file was updated:'
      endif
      write(*,*) res_file(1:strlen(res_file))

      return
      end



      subroutine record_flux_dn_signal(pass,Nk,nu,Nlev,alt,pres,
     &     flux_dn_signal)
      implicit none
      include 'max.inc'
      include 'formats.inc'
c
c     Purpose: to append data to the downward flux signal output file
c
c     Input:
c       + pass: index of the pass over spectral discretization
c       + Nk: number of values of nu
c       + nu: these "Nk" values of nu
c       + Nlev: number of levels (interfaces)
c       + alt: altitude levels (meters)
c       + pres: pressure levels (atm)
c       + flux_dn_signal: downward flux for these "Nk" values
c
c     Output:
c
c     I/O
      integer pass,Nk
      double precision nu(1:Nk_mx)
      integer Nlev
      double precision alt(1:Nlev_mx)
      double precision pres(1:Nlev_mx)
      double precision flux_dn_signal(1:Nk_mx,0:Nlay_mx)
c     temp
      integer Nlay,i,nl,ik,ik1
      character*(Nchar_mx) nlines_file
      character*(Nchar_mx) res_file,line
c     label
      integer strlen
      character*(Nchar_mx) label
      label='subroutine record_flux_dn_signal'

      Nlay=Nlev-1
      res_file='./results/flux_dn_signal.txt'
      line='nu (cm^-1) / F- (W/m^2)'
      nlines_file='./nlines'
c      call get_nlines(res_file,nlines_file,nl)

      open(14,file=res_file(1:strlen(res_file)),access='append')
c      if (nl.gt.0) then
c         do i=1,nl
c            read(14,*)
c         enddo
c      endif
      if (pass.eq.1) then
         ik1=1
         write(14,10) 'Number of levels:'
         write(14,*) Nlev
         write(14,10) 'Altitudes (meters):'
         write(14,52) (alt(i),i=1,Nlev)
         write(14,10) 'Pressures (atm):'
         write(14,52) (pres(i),i=1,Nlev)
         write(14,*)
         write(14,10) line(1:strlen(line))
      else
         ik1=2
      endif
      do ik=ik1,Nk
         write(14,52) nu(ik),(flux_dn_signal(ik,i),i=0,Nlay)
      enddo                     ! ik
      close(14)
      write(*,*) '---'
      if (pass.eq.1) then
         write(*,*) 'Output file was generated:'
      else
         write(*,*) 'Output file was updated:'
      endif
      write(*,*) res_file(1:strlen(res_file))

      return
      end



      subroutine record_flux_net_signal(pass,Nk,nu,Nlev,alt,pres,
     &     flux_net_signal)
      implicit none
      include 'max.inc'
      include 'formats.inc'
c
c     Purpose: to append data to the net flux signal output file
c
c     Input:
c       + pass: index of the pass over spectral discretization
c       + Nk: number of values of nu
c       + nu: these "Nk" values of nu
c       + Nlev: number of levels (interfaces)
c       + alt: altitude levels (meters)
c       + pres: pressure levels (atm)
c       + flux_net_signal: net flux for these "Nk" values
c
c     Output:
c
c     I/O
      integer pass,Nk
      double precision nu(1:Nk_mx)
      integer Nlev
      double precision alt(1:Nlev_mx)
      double precision pres(1:Nlev_mx)
      double precision flux_net_signal(1:Nk_mx,0:Nlay_mx)
c     temp
      integer Nlay,i,nl,ik,ik1
      character*(Nchar_mx) nlines_file
      character*(Nchar_mx) res_file,line
c     label
      integer strlen
      character*(Nchar_mx) label
      label='subroutine record_flux_net_signal'

      Nlay=Nlev-1
      res_file='./results/flux_net_signal.txt'
      line='nu (cm^-1) / Fnet (W/m^2)'
      nlines_file='./nlines'
c      call get_nlines(res_file,nlines_file,nl)

      open(14,file=res_file(1:strlen(res_file)),access='append')
c      if (nl.gt.0) then
c         do i=1,nl
c            read(14,*)
c         enddo
c      endif
      if (pass.eq.1) then
         ik1=1
         write(14,10) 'Number of levels:'
         write(14,*) Nlev
         write(14,10) 'Altitudes (meters):'
         write(14,52) (alt(i),i=1,Nlev)
         write(14,10) 'Pressures (atm):'
         write(14,52) (pres(i),i=1,Nlev)
         write(14,*)
         write(14,10) line(1:strlen(line))
      else
         ik1=2
      endif
      do ik=ik1,Nk
         write(14,52) nu(ik),(flux_net_signal(ik,i),i=0,Nlay)
      enddo                     ! ik
      close(14)
      write(*,*) '---'
      if (pass.eq.1) then
         write(*,*) 'Output file was generated:'
      else
         write(*,*) 'Output file was updated:'
      endif
      write(*,*) res_file(1:strlen(res_file))

      return
      end



      subroutine record_ner(ai,Nlev,nu_min,nu_max,total_NER)
      implicit none
      include 'max.inc'
      include 'formats.inc'
c
c     Purpose: to record the total NER matrix
c
c     Input:
c       + ai: true if angular integration is enabled
c       + Nlev: number of levels
c       + nu_min: lower integration wavenumber (cm^-1)
c       + nu_max: higher integration wavenumber (cm^-1)
c       + total_NER: total NER matrix
c
c     Output:
c
c     I/O
      logical ai
      integer Nlev
      double precision nu_min,nu_max
      double precision total_NER(0:Nlay_mx+1,0:Nlay_mx+1)
c     functions
      double precision oom
c     temp
      integer i,j,Nlay
      integer err
      character*(Nchar_mx) data_file1
      character*(Nchar_mx) data_file2
      character*(Nchar_mx) result_file
      character*(Nchar_mx) gra_file
      character*2 Nlay_str2,Nlayp1_str2
      character*3 Nlay_str3,Nlayp1_str3
      character*(Nchar_mx) Nlay_str,Nlayp1_str
      character*(Nchar_mx) title_str
      double precision si_NER_max,max_oom,min_oom
      character*(Nchar_mx) si_NER_max_str
      character*(Nchar_mx) max_oom_str
      character*(Nchar_mx) min_oom_str
      character*(Nchar_mx) nu_min_str
      character*(Nchar_mx) nu_max_str
c     label
      integer strlen
      character*(Nchar_mx) label
      label='subroutine record_ner'

c     Data file for positive values
      data_file1='si_NER1.dat'
      result_file='./results/'//data_file1(1:strlen(data_file1))
      Nlay=Nlev-1

      open(15,file=result_file(1:strlen(result_file)))
      do i=0,Nlay+1
         do j=0,Nlay+1
            if (total_NER(i,j).gt.0.0D+0) then
               write(15,*) i,j,total_NER(i,j)
            else
               write(15,*) i,j,0.0D+0
            endif
         enddo ! j
         write(15,*)
      enddo ! i
      close(15)
      write(*,*) '---'
      write(*,*) 'Output file was generated:'
      write(*,*) result_file(1:strlen(result_file))
c     Data file for negative values
      data_file2='si_NER2.dat'
      result_file='./results/'//data_file2(1:strlen(data_file2))
      Nlay=Nlev-1

      open(15,file=result_file(1:strlen(result_file)))
      do i=0,Nlay+1
         do j=0,Nlay+1
            if (total_NER(i,j).lt.0.0D+0) then
               write(15,*) i,j,total_NER(i,j)
            else
               write(15,*) i,j,0.0D+0
            endif
         enddo ! j
         write(15,*)
      enddo ! i
      close(15)
      write(*,*) '---'
      write(*,*) 'Output file was generated:'
      write(*,*) result_file(1:strlen(result_file))

c     GNUPLOT script files
      if (Nlay.lt.10) then
         call error(label)
         write(*,*) 'Cannot convert Nlay to character string'
         write(*,*) 'Nlay=',Nlay
         write(*,*) '< 10'
         stop
      else if (Nlay.lt.100) then
         call num2str2(Nlay,Nlay_str2)
         Nlay_str=Nlay_str2(1:strlen(Nlay_str2))
      else if (Nlay.lt.1000) then
         call num2str3(Nlay,Nlay_str3)
         Nlay_str=Nlay_str3(1:strlen(Nlay_str3))
      else
         call error(label)
         write(*,*) 'Cannot convert Nlay to character string'
         write(*,*) 'Nlay=',Nlay
         write(*,*) '> 1000'
         stop
      endif
      if (Nlay+1.lt.10) then
         call error(label)
         write(*,*) 'Cannot convert Nlay+1 to character string'
         write(*,*) 'Nlay+1=',Nlay+1
         write(*,*) '< 10'
         stop
      else if (Nlay+1.lt.100) then
         call num2str2(Nlay+1,Nlayp1_str2)
         Nlayp1_str=Nlayp1_str2(1:strlen(Nlayp1_str2))
      else if (Nlay+1.lt.1000) then
         call num2str3(Nlay+1,Nlayp1_str3)
         Nlayp1_str=Nlayp1_str3(1:strlen(Nlayp1_str3))
      else
         call error(label)
         write(*,*) 'Cannot convert Nlay+1 to character string'
         write(*,*) 'Nlay+1=',Nlay+1
         write(*,*) '> 1000'
         stop
      endif

c     Looking for maximum spectrally integrated NER value (only in gas-gas net exchanges)
      si_NER_max=0.0D+0
      do i=1,Nlay
         do j=1,Nlay
            if (total_NER(i,j).gt.si_NER_max) then
               si_NER_max=total_NER(i,j)
            endif
         enddo                  ! j
      enddo                     ! i
      max_oom=oom(si_NER_max)
      min_oom=max_oom/1.0D+3
c     Debug
c      write(*,*) 'max_oom=',max_oom
c      write(*,*) 'min_oom=',min_oom
c     Debug

      gra_file='./results/gra_si_NER1'
      open(10,file=gra_file(1:strlen(gra_file)))
      write(10,10) 'set term pdf enhanced color font'
     &     //' "helvetica,20" linewidth 1 size 10,6'
      write(10,10) 'set output "si_NER1.pdf"'
      write(10,10) 'set grid layerdefault'
      write(10,10) 'set size square'
      write(10,10) 'set xlabel "layer index i"'
      write(10,10) 'set ylabel "layer index j"'
      write(10,10) 'set palette defined (0 "white", 1 "khaki",'
     &     //' 2 "yellow", 3 "orange", 4 "red")'
c      write(10,10) 'set palette defined (0 "navy", 1 "blue", '
c     &     //'2 "skyblue", 3 "turquoise", 4 "white", '
c     &     //'5 "khaki", 6 "yellow", 7 "orange", 8 "red")'
      write(10,10) 'set pm3d'
      write(10,10) 'set view map'
      write(10,10) 'set xrange [0:'
     &     //Nlayp1_str(1:strlen(Nlayp1_str))
     &     //']'
      write(10,10) 'set yrange [0:'
     &     //Nlayp1_str(1:strlen(Nlayp1_str))
     &     //']'
      call dble2str(min_oom,3,min_oom_str,err)
      if (err.ne.0) then
         call error(label)
         write(*,*) 'Error while converting to character string:'
         write(*,*) 'min_oom=',min_oom
         stop
      endif
c     Debug
c      write(*,*) 'min_oom_str=',min_oom_str
c     Debug
      call dble2str(max_oom,3,max_oom_str,err)
      if (err.ne.0) then
         call error(label)
         write(*,*) 'Error while converting to character string:'
         write(*,*) 'max_oom=',max_oom
         stop
      endif
c     Debug
c      write(*,*) 'max_oom_str=',max_oom_str
c     Debug
      call dble2str(si_NER_max,3,si_NER_max_str,err)
      if (err.ne.0) then
         call error(label)
         write(*,*) 'Error while converting to character string:'
         write(*,*) 'si_NER_max=',si_NER_max
         stop
      endif
      write(10,10) 'set logscale cb'
      write(10,10) 'set cbrange ['
     &     //min_oom_str(1:strlen(min_oom_str))
     &     //':'
     &     //si_NER_max_str(1:strlen(si_NER_max_str))
     &     //']'
      call dble2str_noexp(nu_min,3,nu_min_str,err)
      if (err.ne.0) then
         call error(label)
         write(*,*) 'Error while converting to character string:'
         write(*,*) 'nu_min=',nu_min
         stop
      endif
      call dble2str_noexp(nu_max,3,nu_max_str,err)
      if (err.ne.0) then
         call error(label)
         write(*,*) 'Error while converting to character string:'
         write(*,*) 'nu_max=',nu_max
         stop
      endif
      if (ai) then
         title_str='spectrally integrated NER matrix (W/m^{2})'
      else
         title_str='spectrally integrated NER matrix (W/m^{2}/sr)'
      endif
      title_str=title_str(1:strlen(title_str))
     &     //'\n{/Symbol n}=['
     &     //nu_min_str(1:strlen(nu_min_str))
     &     //','
     &     //nu_max_str(1:strlen(nu_max_str))
     &     //'] cm^{-1}'
      write(10,10) 'set title "'
     &     //title_str(1:strlen(title_str))
     &     //'"'
      write(10,10) 'plot "./'
     &     //data_file1(1:strlen(data_file1))
     &     //'" u 1:2:3 w image notitle'
      write(10,*)
      write(10,10) 'quit'
      close(10)
c      
c      
c      
      gra_file='./results/gra_si_NER2'
      open(10,file=gra_file(1:strlen(gra_file)))
      write(10,10) 'set term pdf enhanced color font'
     &     //' "helvetica,20" linewidth 1 size 10,6'
      write(10,10) 'set output "si_NER2.pdf"'
      write(10,10) 'set grid layerdefault'
      write(10,10) 'set size square'
      write(10,10) 'set xlabel "layer index i"'
      write(10,10) 'set ylabel "layer index j"'
      write(10,10) 'set palette defined (0 "white", 1 "turquoise",'
     &     //' 2 "skyblue", 3 "blue", 4 "navy")'
c      write(10,10) 'set palette defined (0 "navy", 1 "blue", '
c     &     //'2 "skyblue", 3 "turquoise", 4 "white", '
c     &     //'5 "khaki", 6 "yellow", 7 "orange", 8 "red")'
      write(10,10) 'set pm3d'
      write(10,10) 'set view map'
      write(10,10) 'set xrange [0:'
     &     //Nlayp1_str(1:strlen(Nlayp1_str))
     &     //']'
      write(10,10) 'set yrange [0:'
     &     //Nlayp1_str(1:strlen(Nlayp1_str))
     &     //']'
      call dble2str(min_oom,3,min_oom_str,err)
      if (err.ne.0) then
         call error(label)
         write(*,*) 'Error while converting to character string:'
         write(*,*) 'min_oom=',min_oom
         stop
      endif
      call dble2str(max_oom,3,max_oom_str,err)
      if (err.ne.0) then
         call error(label)
         write(*,*) 'Error while converting to character string:'
         write(*,*) 'max_oom=',max_oom
         stop
      endif
      call dble2str(si_NER_max,3,si_NER_max_str,err)
      if (err.ne.0) then
         call error(label)
         write(*,*) 'Error while converting to character string:'
         write(*,*) 'si_NER_max=',si_NER_max
         stop
      endif
      write(10,10) 'set logscale cb'
      write(10,10) 'set cbrange ['
     &     //min_oom_str(1:strlen(min_oom_str))
     &     //':'
     &     //si_NER_max_str(1:strlen(si_NER_max_str))
     &     //']'
      call dble2str_noexp(nu_min,3,nu_min_str,err)
      if (err.ne.0) then
         call error(label)
         write(*,*) 'Error while converting to character string:'
         write(*,*) 'nu_min=',nu_min
         stop
      endif
      call dble2str_noexp(nu_max,3,nu_max_str,err)
      if (err.ne.0) then
         call error(label)
         write(*,*) 'Error while converting to character string:'
         write(*,*) 'nu_max=',nu_max
         stop
      endif
      if (ai) then
         title_str='spectrally integrated NER matrix (W/m^{2})'
      else
         title_str='spectrally integrated NER matrix (W/m^{2}/sr)'
      endif
      title_str=title_str(1:strlen(title_str))
     &     //'\n{/Symbol n}=['
     &     //nu_min_str(1:strlen(nu_min_str))
     &     //','
     &     //nu_max_str(1:strlen(nu_max_str))
     &     //'] cm^{-1}'
      write(10,10) 'set title "'
     &     //title_str(1:strlen(title_str))
     &     //'"'
      write(10,10) 'plot "./'
     &     //data_file2(1:strlen(data_file2))
     &     //'" u 1:2:(-1.0*$3) w image notitle'
      write(10,*)
      write(10,10) 'quit'
      close(10)

      return
      end



      double precision function oom(value)
      implicit none
      include 'max.inc'
c     
c     Purpose: to get the order of magnitude of a given value
c     
c     Input:
c       + value: double precision value
c     
c     Output:
c       + oom: order of magnitude
c
c     I/O
      double precision value
c     temp
      double precision t1
      integer i1
c     label
      integer strlen
      character*(Nchar_mx) label
      label='function oom'

      t1=dlog10(value)
      i1=int(t1)
      if (t1.lt.0.0D+0) then
         i1=i1-1
      endif
      oom=10**dble(i1)

      return
      end
      



      subroutine record_flux_up(ai,Nlev,alt,pres,total_flux_up)
      implicit none
      include 'max.inc'
      include 'formats.inc'
c
c     Purpose: to record total fluxes
c
c     Input:
c       + ai: true if angular integration is enabled
c       + Nlev: number of levels
c       + alt: altitude levels (meters)
c       + pres: pressure levels (atm)
c       + total_flux_up: total upward flux profile (W/m^2)
c
c     Output:
c
c     I/O
      logical ai
      integer Nlev
      double precision alt(1:Nlev_mx)
      double precision pres(1:Nlev_mx)
      double precision total_flux_up(0:Nlay_mx)
c     temp
      integer i,Nlay
      character*(Nchar_mx) res_file,gra_file,line
c     label
      integer strlen
      character*(Nchar_mx) label
      label='subroutine record_flux_up'

      Nlay=Nlev-1

      res_file='./results/flux_up.txt'
      line='altitude (m) / pressure (atm) / F+ (W/m^2)'
      open(16,file=res_file(1:strlen(res_file)))
      write(16,10) line(1:strlen(line))
      do i=0,Nlay
         write(16,52) alt(i+1),pres(i+1),total_flux_up(i)
      enddo ! i
      close(16)
      write(*,*) '---'
      write(*,*) 'Output file was generated:'
      write(*,*) res_file(1:strlen(res_file))
c
      gra_file='./results/gra_si_flux'
      open(10,file=gra_file(1:strlen(gra_file)))
      write(10,10) 'set term pdf enhanced color font "helvetica,20" '
     &     //'linewidth 1 size 10,6'
      write(10,10) 'set output "si_flux.pdf"'
      if (ai) then
         write(10,10) 'set xlabel "Flux (W/m^{2})"'
      else
         write(10,10) 'set xlabel "Flux (W/m^{2}/sr)"'
      endif
      write(10,10) 'set ylabel "Altitude (km)"'
      write(10,10) 'plot \'
      write(10,10) '     "./flux_up.txt" u 3:($1/1000) title '
     &     //'"upward" w lines lt -1 lw 1 \'
      write(10,10) '     ,"./flux_dn.txt" u 3:($1/1000) title '
     &     //'"downward" w lines lt 0 lw 1 \'
      write(10,10)  '    ,"./flux_net.txt" u 3:($1/1000) title '
     &     //'"net" w lines lt 1 lw 1 \'
      write(10,10)
      write(10,10) 'quit'
      close(10)

      return
      end



      subroutine record_flux_dn(ai,Nlev,alt,pres,total_flux_dn)
      implicit none
      include 'max.inc'
      include 'formats.inc'
c
c     Purpose: to record total fluxes
c
c     Input:
c       + ai: true if angular integration is enabled
c       + Nlev: number of levels
c       + alt: altitude levels (meters)
c       + pres: pressure levels (atm)
c       + total_flux_dn: total downward flux profile (W/m^2)
c
c     Output:
c
c     I/O
      logical ai
      integer Nlev
      double precision alt(1:Nlev_mx)
      double precision pres(1:Nlev_mx)
      double precision total_flux_dn(0:Nlay_mx)
c     temp
      integer i,Nlay
      character*(Nchar_mx) res_file,line
c     label
      integer strlen
      character*(Nchar_mx) label
      label='subroutine record_flux_dn'

      Nlay=Nlev-1

      res_file='./results/flux_dn.txt'
      line='altitude (m) / pressure (atm) / F- (W/m^2)'
      open(17,file=res_file(1:strlen(res_file)))
      write(17,10) line(1:strlen(line))
      do i=0,Nlay
         write(17,52) alt(i+1),pres(i+1),total_flux_dn(i)
      enddo ! i
      close(17)
      write(*,*) '---'
      write(*,*) 'Output file was generated:'
      write(*,*) res_file(1:strlen(res_file))

      return
      end



      subroutine record_flux_net(ai,Nlev,alt,pres,total_flux_net)
      implicit none
      include 'max.inc'
      include 'formats.inc'
c
c     Purpose: to record total fluxes
c
c     Input:
c       + ai: true if angular integration is enabled
c       + Nlev: number of levels
c       + alt: altitude levels (meters)
c       + pres: pressure levels (atm)
c       + total_flux_net: total net flux profile (W/m^2)
c
c     Output:
c
c     I/O
      logical ai
      integer Nlev
      double precision alt(1:Nlev_mx)
      double precision pres(1:Nlev_mx)
      double precision total_flux_net(0:Nlay_mx)
c     temp
      integer i,Nlay
      character*(Nchar_mx) res_file,line
c     label
      integer strlen
      character*(Nchar_mx) label
      label='subroutine record_flux_net'

      Nlay=Nlev-1

      res_file='./results/flux_net.txt'
      line='altitude (m) / pressure (atm) / Fnet (W/m^2)'
      open(18,file=res_file(1:strlen(res_file)))
      write(18,10) line(1:strlen(line))
      do i=0,Nlay
         write(18,52) alt(i+1),pres(i+1),total_flux_net(i)
      enddo ! i
      close(18)
      write(*,*) '---'
      write(*,*) 'Output file was generated:'
      write(*,*) res_file(1:strlen(res_file))

      return
      end



      subroutine record_total_pflux(ai,probe_pressure,probe_altitude,
     &     total_pflux_up,total_pflux_dn,total_pflux_net)
      implicit none
      include 'max.inc'
      include 'formats.inc'
c
c     Purpose: to record total fluxes
c
c     Input:
c       + ai: true if angular integration is enabled
c       + probe_pressure: probe pressure (atm)
c       + probe_altitude: corresponding altitude (m)
c       + total_pflux_up: total upward flux @ probe pressure (W/m^2)
c       + total_pflux_dn: total downward flux @ probe pressure (W/m^2)
c       + total_pflux_net: total net flux @ probe pressure (W/m^2)
c
c     Output:
c
c     I/O
      logical ai
      double precision probe_pressure
      double precision probe_altitude
      double precision total_pflux_up
      double precision total_pflux_dn
      double precision total_pflux_net
c     temp
      character*(Nchar_mx) res_file,line
c     label
      integer strlen
      character*(Nchar_mx) label
      label='subroutine record_flux_net'

      res_file='./results/total_probe_flux.txt'
      line='probe pressure (atm) / probe altitude (m) /  '
     &     //'Fup (W/m^2) /  Fdn (W/m^2) / Fnet (W/m^2)'
      open(18,file=res_file(1:strlen(res_file)))
      write(18,10) line(1:strlen(line))
      write(18,52) probe_pressure,
     &     probe_altitude,
     &     total_pflux_up,
     &     total_pflux_dn,
     &     total_pflux_net
      close(18)
      write(*,*) '---'
      write(*,*) 'Output file was generated:'
      write(*,*) res_file(1:strlen(res_file))

      return
      end



      subroutine record_budget_signal(pass,Nk,nu,Nlev,alt,pres,capa,
     &     budget_signal)
      implicit none
      include 'max.inc'
      include 'formats.inc'
c
c     Purpose: to record radiative budget signal
c
c     Input:
c       + pass: index of the pass over spectral discretization
c       + Nk: number of values of nu
c       + nu: these "Nk" values of nu
c       + Nlev: number of levels
c       + alt: altitude levels (meters)
c       + pres: pressure levels (atm)
c       + capa: average calorific capacity of each layer [J/K/m^3/(time_period)]
c       + budget_signal: radiative budget signal (W/m^2)
c
c     Output:
c
c     I/O
      integer pass,Nk
      double precision nu(1:Nk_mx)
      integer Nlev
      double precision alt(1:Nlev_mx)
      double precision pres(1:Nlev_mx)
      double precision capa(1:Nlay_mx)
      double precision budget_signal(1:Nk_mx,1:Nlay_mx)
c     temp
      integer i,Nlay,ik,ik1
      character*(Nchar_mx) res_file,line
c     label
      integer strlen
      character*(Nchar_mx) label
      label='subroutine record_budget_signal'

      Nlay=Nlev-1

      res_file='./results/budget_signal.txt'
      line='nu (cm^-1) / budget (W/m^2) / budget (W/m^3)'
      open(19,file=res_file(1:strlen(res_file)),access='append')
      if (pass.eq.1) then
         ik1=1
         write(19,10) 'Number of layers:'
         write(19,*) Nlay
         write(19,10) 'Altitudes (meters):'
         write(19,52) (alt(i),i=1,Nlev)
         write(19,10) 'Pressures (atm):'
         write(19,52) (pres(i),i=1,Nlev)
         write(19,*)
         write(19,10) line(1:strlen(line))
      else
         ik1=2
      endif
      do ik=ik1,Nk
         write(19,52) nu(ik),
     &        (budget_signal(ik,i),i=1,Nlay),
     &        (budget_signal(ik,i)/(alt(i+1)-alt(i)),i=1,Nlay)
      enddo                     ! ik
      close(19)
      write(*,*) '---'
      if (pass.eq.1) then
         write(*,*) 'Output file was generated:'
      else
         write(*,*) 'Output file was updated:'
      endif
      write(*,*) res_file(1:strlen(res_file))
c     
      res_file='./results/cooling_rate_signal.txt'
      line='nu (cm^-1) / cooling rate [K/time_period] '
     &     //'(positive means heating)'
      open(19,file=res_file(1:strlen(res_file)),access='append')
      if (pass.eq.1) then
         ik1=1
         write(19,10) 'Number of layers:'
         write(19,*) Nlay
         write(19,10) 'Altitudes (meters):'
         write(19,52) (alt(i),i=1,Nlev)
         write(19,10) 'Pressures (atm):'
         write(19,52) (pres(i),i=1,Nlev)
         write(19,*)
         write(19,10) line(1:strlen(line))
      else
         ik1=2
      endif
      do ik=ik1,Nk
         write(19,52) nu(ik),
     &        (budget_signal(ik,i)/(alt(i+1)-alt(i))/capa(i),i=1,Nlay) ! K/[time_period]
      enddo                     ! ik
      close(19)
      write(*,*) '---'
      if (pass.eq.1) then
         write(*,*) 'Output file was generated:'
      else
         write(*,*) 'Output file was updated:'
      endif
      write(*,*) res_file(1:strlen(res_file))

      return
      end



      subroutine record_ES_signal(pass,Nk,nu,
     &     ground_space_NER_signal,flux_up_signal,
     &     tau_total_cumulated)
      implicit none
      include 'max.inc'
      include 'formats.inc'
c
c     Purpose: to record radiative ground-space budget signal
c
c     Input:
c       + pass: index of the pass over spectral discretization
c       + Nk: number of values of nu
c       + nu: these "Nk" values of nu
c       + ground_space_NER_signal: radiative ground-space budget signal (W/m^2)
c       + flux_up_signal: upward flux for these "Nk" values
c       + tau_total_cumulated: cumlated total optical depth
c
c     Output:
c
c     I/O
      integer pass,Nk
      double precision nu(1:Nk_mx)
      double precision ground_space_NER_signal(1:Nk_mx)
      double precision flux_up_signal(1:Nk_mx,0:Nlay_mx)
      double precision tau_total_cumulated(1:Nk_mx)
c     temp
      integer ik,ik1
      character*(Nchar_mx) res_file,line
c     functions
      double precision expint
c     label
      integer strlen
      character*(Nchar_mx) label
      label='subroutine record_ground_space_NER_signal'

      res_file='./results/ES_signal.txt'
      line='nu (cm^-1) / flux emitted by the ground (W/m^2/cm^-1) /'
     &     //' ground-space NER (W/m^2/cm^-1) /'
     &     //' optical depth / analytic transmittivity / '
     &     //' transmittivity from radiative transfer'
      open(19,file=res_file(1:strlen(res_file)),access='append')
      if (pass.eq.1) then
         ik1=1
         write(19,10) line(1:strlen(line))
      else
         ik1=2
      endif
      do ik=ik1,Nk
         write(19,52) nu(ik)
     &        ,flux_up_signal(ik,0)
     &        ,ground_space_NER_signal(ik)
     &        ,tau_total_cumulated(ik)
c     The last two columns hold the fraction of ground emission that reaches space:
     &        ,2.0D+0*expint(3,tau_total_cumulated(ik))
c     This is the analytical form; the factor "2" comes from the fact that for a lambertian ground emission Bg,
c     the flux emitted by the ground is pi*Bg and the flux at TOA is 2*pi*Bg*E3(tau); the total transmission
c     is therefore 2*pi*Bg*E3(tau)/(pi*Bg)=2*E3(tau)
c     
     &        ,ground_space_NER_signal(ik)/flux_up_signal(ik,0)
c     This is the result obtained from computation results: the NER between ground and space is indeed
c     the fraction of the flux emitted by the ground (i.e. flux_up_signal(ik,0) ) that reaches space.
c     
      enddo                     ! ik
      close(19)
      write(*,*) '---'
      if (pass.eq.1) then
         write(*,*) 'Output file was generated:'
      else
         write(*,*) 'Output file was updated:'
      endif
      write(*,*) res_file(1:strlen(res_file))

      return
      end



      subroutine record_ground_space_NER_signal(pass,Nk,nu,
     &     ground_space_NER_signal,flux_up_signal,
     &     tau_total_cumulated)
      implicit none
      include 'max.inc'
      include 'formats.inc'
c
c     Purpose: to record radiative ground-space budget signal
c
c     Input:
c       + pass: index of the pass over spectral discretization
c       + Nk: number of values of nu
c       + nu: these "Nk" values of nu
c       + ground_space_NER_signal: radiative ground-space budget signal (W/m^2)
c       + flux_up_signal: upward flux for these "Nk" values
c       + tau_total_cumulated: cumlated total optical depth
c
c     Output:
c
c     I/O
      integer pass,Nk
      double precision nu(1:Nk_mx)
      integer Nlev
      double precision ground_space_NER_signal(1:Nk_mx)
      double precision flux_up_signal(1:Nk_mx,0:Nlay_mx)
      double precision tau_total_cumulated(1:Nk_mx)
c     temp
      integer ik,ik1
      character*(Nchar_mx) res_file,line
c     functions
      double precision expint
c     label
      integer strlen
      character*(Nchar_mx) label
      label='subroutine record_ground_space_NER_signal'

      res_file='./results/ground_space_NER_signal.txt'
      line='nu (cm^-1) / ground-space et exchange (W/m^2/cm^-1)'
      open(19,file=res_file(1:strlen(res_file)),access='append')
      if (pass.eq.1) then
         ik1=1
         write(19,10) line(1:strlen(line))
      else
         ik1=2
      endif
      do ik=ik1,Nk
         write(19,52) nu(ik),
     &        ground_space_NER_signal(ik)
      enddo                     ! ik
      close(19)
      write(*,*) '---'
      if (pass.eq.1) then
         write(*,*) 'Output file was generated:'
      else
         write(*,*) 'Output file was updated:'
      endif
      write(*,*) res_file(1:strlen(res_file))

      return
      end
      


      subroutine record_gas_ground_budget_signal(pass,Nk,nu,
     &     Nlev,alt,pres,capa,gas_ground_budget_signal)
      implicit none
      include 'max.inc'
      include 'formats.inc'
c
c     Purpose: to record radiative gas-ground budget signal
c
c     Input:
c       + pass: index of the pass over spectral discretization
c       + Nk: number of values of nu
c       + nu: these "Nk" values of nu
c       + Nlev: number of levels
c       + alt: altitude levels (meters)
c       + pres: pressure levels (atm)
c       + capa: average calorific capacity of each layer [J/K/m^3/(time_period)]
c       + gas_ground_budget_signal: radiative gas-ground budget signal (W/m^2)
c
c     Output:
c
c     I/O
      integer pass,Nk
      double precision nu(1:Nk_mx)
      integer Nlev
      double precision alt(1:Nlev_mx)
      double precision pres(1:Nlev_mx)
      double precision capa(1:Nlay_mx)
      double precision gas_ground_budget_signal(1:Nk_mx,1:Nlay_mx)
c     temp
      integer i,Nlay,ik,ik1
      character*(Nchar_mx) res_file,line
c     label
      integer strlen
      character*(Nchar_mx) label
      label='subroutine record_gas_ground_budget_signal'

      Nlay=Nlev-1

      res_file='./results/gas_ground_budget_signal.txt'
      line='nu (cm^-1) / budget (W/m^2) / budget (W/m^3)'
      open(19,file=res_file(1:strlen(res_file)),access='append')
      if (pass.eq.1) then
         ik1=1
         write(19,10) 'Number of layers:'
         write(19,*) Nlay
         write(19,10) 'Altitudes (meters):'
         write(19,52) (alt(i),i=1,Nlev)
         write(19,10) 'Pressures (atm):'
         write(19,52) (pres(i),i=1,Nlev)
         write(19,*)
         write(19,10) line(1:strlen(line))
      else
         ik1=2
      endif
      do ik=ik1,Nk
         write(19,52) nu(ik),
     &        (gas_ground_budget_signal(ik,i),i=1,Nlay),
     &        (gas_ground_budget_signal(ik,i)
     &        /(alt(i+1)-alt(i)),i=1,Nlay)
      enddo                     ! ik
      close(19)
      write(*,*) '---'
      if (pass.eq.1) then
         write(*,*) 'Output file was generated:'
      else
         write(*,*) 'Output file was updated:'
      endif
      write(*,*) res_file(1:strlen(res_file))
c
      res_file='./results/gas_ground_coolingrate_signal.txt'
      line='nu (cm^-1) / cooling rate [K/(time_period)] '
     &     //'(positive means heating)'
      open(19,file=res_file(1:strlen(res_file)),access='append')
      if (pass.eq.1) then
         ik1=1
         write(19,10) 'Number of layers:'
         write(19,*) Nlay
         write(19,10) 'Altitudes (meters):'
         write(19,52) (alt(i),i=1,Nlev)
         write(19,10) 'Pressures (atm):'
         write(19,52) (pres(i),i=1,Nlev)
         write(19,*)
         write(19,10) line(1:strlen(line))
      else
         ik1=2
      endif
      do ik=ik1,Nk
         write(19,52) nu(ik),
     &        (gas_ground_budget_signal(ik,i)
     &        /(alt(i+1)-alt(i))/capa(i),i=1,Nlay)
      enddo                     ! ik
      close(19)
      write(*,*) '---'
      if (pass.eq.1) then
         write(*,*) 'Output file was generated:'
      else
         write(*,*) 'Output file was updated:'
      endif
      write(*,*) res_file(1:strlen(res_file))

      return
      end



      subroutine record_gas_space_budget_signal(pass,Nk,nu,
     &     Nlev,alt,pres,capa,gas_space_budget_signal)
      implicit none
      include 'max.inc'
      include 'formats.inc'
c
c     Purpose: to record radiative gas-space budget signal
c
c     Input:
c       + pass: index of the pass over spectral discretization
c       + Nk: number of values of nu
c       + nu: these "Nk" values of nu
c       + Nlev: number of levels
c       + alt: altitude levels (meters)
c       + pres: pressure levels (atm)
c       + capa: average calorific capacity of each layer [J/K/m^3/(time_period)]
c       + gas_space_budget_signal: radiative gas-space budget signal (W/m^2)
c
c     Output:
c
c     I/O
      integer pass,Nk
      double precision nu(1:Nk_mx)
      integer Nlev
      double precision alt(1:Nlev_mx)
      double precision pres(1:Nlev_mx)
      double precision capa(1:Nlay_mx)
      double precision gas_space_budget_signal(1:Nk_mx,1:Nlay_mx)
c     temp
      integer i,Nlay,ik,ik1
      character*(Nchar_mx) res_file,line
c     label
      integer strlen
      character*(Nchar_mx) label
      label='subroutine record_gas_space_budget_signal'

      Nlay=Nlev-1

      res_file='./results/gas_space_budget_signal.txt'
      line='nu (cm^-1) / budget (W/m^2) / budget (W/m^3)'
      open(19,file=res_file(1:strlen(res_file)),access='append')
      if (pass.eq.1) then
         ik1=1
         write(19,10) 'Number of layers:'
         write(19,*) Nlay
         write(19,10) 'Altitudes (meters):'
         write(19,52) (alt(i),i=1,Nlev)
         write(19,10) 'Pressures (atm):'
         write(19,52) (pres(i),i=1,Nlev)
         write(19,*)
         write(19,10) line(1:strlen(line))
      else
         ik1=2
      endif
      do ik=ik1,Nk
         write(19,52) nu(ik),
     &        (gas_space_budget_signal(ik,i),i=1,Nlay),
     &        (gas_space_budget_signal(ik,i)/(alt(i+1)-alt(i)),i=1,Nlay)
      enddo                     ! ik
      close(19)
      write(*,*) '---'
      if (pass.eq.1) then
         write(*,*) 'Output file was generated:'
      else
         write(*,*) 'Output file was updated:'
      endif
      write(*,*) res_file(1:strlen(res_file))
c
      res_file='./results/gas_space_coolingrate_signal.txt'
      line='nu (cm^-1) / cooling rate [K/(time_period)] '
     &     //'(positive means heating)'
      open(19,file=res_file(1:strlen(res_file)),access='append')
      if (pass.eq.1) then
         ik1=1
         write(19,10) 'Number of layers:'
         write(19,*) Nlay
         write(19,10) 'Altitudes (meters):'
         write(19,52) (alt(i),i=1,Nlev)
         write(19,10) 'Pressures (atm):'
         write(19,52) (pres(i),i=1,Nlev)
         write(19,*)
         write(19,10) line(1:strlen(line))
      else
         ik1=2
      endif
      do ik=ik1,Nk
         write(19,52) nu(ik),
     &        (gas_space_budget_signal(ik,i)
     &        /(alt(i+1)-alt(i))/capa(i),i=1,Nlay)
      enddo                     ! ik
      close(19)
      write(*,*) '---'
      if (pass.eq.1) then
         write(*,*) 'Output file was generated:'
      else
         write(*,*) 'Output file was updated:'
      endif
      write(*,*) res_file(1:strlen(res_file))

      return
      end



      subroutine record_gas_gas_budget_signal(pass,Nk,nu,
     &     Nlev,alt,pres,capa,gas_gas_budget_signal)
      implicit none
      include 'max.inc'
      include 'formats.inc'
c
c     Purpose: to record radiative gas-gas budget signal
c
c     Input:
c       + pass: index of the pass over spectral discretization
c       + Nk: number of values of nu
c       + nu: these "Nk" values of nu
c       + Nlev: number of levels
c       + alt: altitude levels (meters)
c       + pres: pressure levels (atm)
c       + capa: average calorific capacity of each layer [J/K/m^3/(time_period)]
c       + gas_gas_budget_signal: radiative gas-gas budget signal (W/m^2)
c
c     Output:
c
c     I/O
      integer pass,Nk
      double precision nu(1:Nk_mx)
      integer Nlev
      double precision alt(1:Nlev_mx)
      double precision pres(1:Nlev_mx)
      double precision capa(1:Nlay_mx)
      double precision gas_gas_budget_signal(1:Nk_mx,1:Nlay_mx)
c     temp
      integer i,Nlay,ik,ik1
      character*(Nchar_mx) res_file,line
c     label
      integer strlen
      character*(Nchar_mx) label
      label='subroutine record_gas_gas_budget_signal'

      Nlay=Nlev-1

      res_file='./results/gas_gas_budget_signal.txt'
      line='nu (cm^-1) / budget (W/m^2) / budget (W/m^3)'
      open(19,file=res_file(1:strlen(res_file)),access='append')
      if (pass.eq.1) then
         ik1=1
         write(19,10) 'Number of layers:'
         write(19,*) Nlay
         write(19,10) 'Altitudes (meters):'
         write(19,52) (alt(i),i=1,Nlev)
         write(19,10) 'Pressures (atm):'
         write(19,52) (pres(i),i=1,Nlev)
         write(19,*)
         write(19,10) line(1:strlen(line))
      else
         ik1=2
      endif
      do ik=ik1,Nk
         write(19,52) nu(ik),
     &        (gas_gas_budget_signal(ik,i),i=1,Nlay),
     &        (gas_gas_budget_signal(ik,i)/(alt(i+1)-alt(i)),i=1,Nlay)
      enddo                     ! ik
      close(19)
      write(*,*) '---'
      if (pass.eq.1) then
         write(*,*) 'Output file was generated:'
      else
         write(*,*) 'Output file was updated:'
      endif
      write(*,*) res_file(1:strlen(res_file))
c
      res_file='./results/gas_gas_coolingrate_signal.txt'
      line='nu (cm^-1) / cooling rate [K/(time_period)] '
     &     //'(positive means heating)'
      open(19,file=res_file(1:strlen(res_file)),access='append')
      if (pass.eq.1) then
         ik1=1
         write(19,10) 'Number of layers:'
         write(19,*) Nlay
         write(19,10) 'Altitudes (meters):'
         write(19,52) (alt(i),i=1,Nlev)
         write(19,10) 'Pressures (atm):'
         write(19,52) (pres(i),i=1,Nlev)
         write(19,*)
         write(19,10) line(1:strlen(line))
      else
         ik1=2
      endif
      do ik=ik1,Nk
         write(19,52) nu(ik),
     &        (gas_gas_budget_signal(ik,i)
     &        /(alt(i+1)-alt(i))/capa(i),i=1,Nlay)
      enddo                     ! ik
      close(19)
      write(*,*) '---'
      if (pass.eq.1) then
         write(*,*) 'Output file was generated:'
      else
         write(*,*) 'Output file was updated:'
      endif
      write(*,*) res_file(1:strlen(res_file))

      return
      end



      subroutine record_budget(Nlev,alt,capa,total_budget)
      implicit none
      include 'max.inc'
      include 'formats.inc'
c
c     Purpose: to record total radiative budget
c
c     Input:
c       + Nlev: number of levels
c       + alt: altitude levels (meters)
c       + capa: average calorific capacity of each layer [J/K/m^3/(time_period)]
c       + total_budget: total radiative budget profile (W/m^2)
c
c     Output:
c
c     I/O
      integer Nlev
      double precision alt(1:Nlev_mx)
      double precision capa(1:Nlay_mx)
      double precision total_budget(1:Nlay_mx)
c     temp
      integer i,Nlay
      character*(Nchar_mx) res_file,line
c     label
      integer strlen
      character*(Nchar_mx) label
      label='subroutine record_budget'

      Nlay=Nlev-1

      res_file='./results/budget.txt'
      line='alt. inf. (m) / alt. sup. (m) /'
     &     //' budget (W/m^2) / budget (W/m^3)'
      open(19,file=res_file(1:strlen(res_file)))
      write(19,10) line(1:strlen(line))
      do i=1,Nlay
         write(19,52) alt(i),alt(i+1),
     &        total_budget(i),
     &        total_budget(i)/(alt(i+1)-alt(i))
      enddo ! i
      close(19)
      write(*,*) '---'
      write(*,*) 'Output file was generated:'
      write(*,*) res_file(1:strlen(res_file))
c
      res_file='./results/cooling_rate.txt'
      line='alt. inf. (m) / alt. sup. (m) /'
     &     //' cooling rate [K/time_period] '
     &     //'(positive means heating)'
      open(19,file=res_file(1:strlen(res_file)))
      write(19,10) line(1:strlen(line))
      do i=1,Nlay
         write(19,52) alt(i),alt(i+1),
     &        total_budget(i)/(alt(i+1)-alt(i))/capa(i)
      enddo ! i
      close(19)
      write(*,*) '---'
      write(*,*) 'Output file was generated:'
      write(*,*) res_file(1:strlen(res_file))

      return
      end



      subroutine record_gas_ground_budget(Nlev,alt,capa,
     &     total_gas_ground_budget)
      implicit none
      include 'max.inc'
      include 'formats.inc'
c
c     Purpose: to record the part of the total radiative budget
c     that is due to gas-ground net exchanges
c
c     Input:
c       + Nlev: number of levels
c       + alt: altitude levels (meters)
c       + capa: average calorific capacity of each layer [J/K/m^3/(time_period)]
c       + total_gas_ground_budget: part of the total radiative budget
c         that is due to gas-ground net exchanges (W/m^2)
c
c     Output:
c
c     I/O
      integer Nlev
      double precision alt(1:Nlev_mx)
      double precision capa(1:Nlay_mx)
      double precision total_gas_ground_budget(1:Nlay_mx)
c     temp
      integer i,Nlay
      character*(Nchar_mx) res_file,line
c     label
      integer strlen
      character*(Nchar_mx) label
      label='subroutine record_gas_ground_budget'

      Nlay=Nlev-1

      res_file='./results/gas_ground_budget.txt'
      line='alt. inf. (m) / alt. sup. (m) /'
     &     //' budget (W/m^2) / budget (W/m^3)'
      open(21,file=res_file(1:strlen(res_file)))
      write(21,10) line(1:strlen(line))
      do i=1,Nlay
         write(21,52) alt(i),alt(i+1),
     &        total_gas_ground_budget(i),
     &        total_gas_ground_budget(i)/(alt(i+1)-alt(i))
      enddo ! i
      close(21)
      write(*,*) '---'
      write(*,*) 'Output file was generated:'
      write(*,*) res_file(1:strlen(res_file))
c
      res_file='./results/gas_ground_coolingrate.txt'
      line='alt. inf. (m) / alt. sup. (m) /'
     &     //' cooling rate [K/(time_period)] '
     &     //'(positive means heating)'
      open(21,file=res_file(1:strlen(res_file)))
      write(21,10) line(1:strlen(line))
      do i=1,Nlay
         write(21,52) alt(i),alt(i+1),
     &        total_gas_ground_budget(i)
     &        /(alt(i+1)-alt(i))/capa(i)
      enddo ! i
      close(21)
      write(*,*) '---'
      write(*,*) 'Output file was generated:'
      write(*,*) res_file(1:strlen(res_file))

      return
      end




      subroutine record_gas_space_budget(Nlev,alt,capa,
     &     total_gas_space_budget)
      implicit none
      include 'max.inc'
      include 'formats.inc'
c
c     Purpose: to record the part of the total radiative budget
c     that is due to gas-space net exchanges
c
c     Input:
c       + Nlev: number of levels
c       + alt: altitude levels (meters)
c       + capa: average calorific capacity of each layer [J/K/m^3/(time_period)]
c       + total_gas_space_budget: part of the total radiative budget
c         that is due to gas-space net exchanges (W/m^2)
c
c     Output:
c
c     I/O
      integer Nlev
      double precision alt(1:Nlev_mx)
      double precision capa(1:Nlay_mx)
      double precision total_gas_space_budget(1:Nlay_mx)
c     temp
      integer i,Nlay
      character*(Nchar_mx) res_file,line
c     label
      integer strlen
      character*(Nchar_mx) label
      label='subroutine record_gas_space_budget'

      Nlay=Nlev-1

      res_file='./results/gas_space_budget.txt'
      line='alt. inf. (m) / alt. sup. (m) /'
     &     //' budget (W/m^2) / budget (W/m^3)'
      open(22,file=res_file(1:strlen(res_file)))
      write(22,10) line(1:strlen(line))
      do i=1,Nlay
         write(22,52) alt(i),alt(i+1),
     &        total_gas_space_budget(i),
     &        total_gas_space_budget(i)/(alt(i+1)-alt(i))
      enddo ! i
      close(22)
      write(*,*) '---'
      write(*,*) 'Output file was generated:'
      write(*,*) res_file(1:strlen(res_file))
c
      res_file='./results/gas_space_coolingrate.txt'
      line='alt. inf. (m) / alt. sup. (m) /'
     &     //' cooling rate [K/(time_period)] '
     &     //'(positive means heating)'
      open(22,file=res_file(1:strlen(res_file)))
      write(22,10) line(1:strlen(line))
      do i=1,Nlay
         write(22,52) alt(i),alt(i+1),
     &        total_gas_space_budget(i)
     &        /(alt(i+1)-alt(i))/capa(i)
      enddo ! i
      close(22)
      write(*,*) '---'
      write(*,*) 'Output file was generated:'
      write(*,*) res_file(1:strlen(res_file))

      return
      end



      subroutine record_gas_gas_budget(Nlev,alt,capa,
     &     total_gas_gas_budget)
      implicit none
      include 'max.inc'
      include 'formats.inc'
c
c     Purpose: to record the part of the total radiative budget
c     that is due to gas-gas net exchanges
c
c     Input:
c       + Nlev: number of levels
c       + alt: altitude levels (meters)
c       + capa: average calorific capacity of each layer [J/K/m^3/(time_period)]
c       + total_gas_gas_budget: part of the total radiative budget
c         that is due to gas-gas net exchanges (W/m^2)
c
c     Output:
c
c     I/O
      integer Nlev
      double precision alt(1:Nlev_mx)
      double precision capa(1:Nlay_mx)
      double precision total_gas_gas_budget(1:Nlay_mx)
c     temp
      integer i,Nlay
      character*(Nchar_mx) res_file,line
c     label
      integer strlen
      character*(Nchar_mx) label
      label='subroutine record_gas_gas_budget'

      Nlay=Nlev-1

      res_file='./results/gas_gas_budget.txt'
      line='alt. inf. (m) / alt. sup. (m) /'
     &     //' budget (W/m^2) / budget (W/m^3)'
      open(23,file=res_file(1:strlen(res_file)))
      write(23,10) line(1:strlen(line))
      do i=1,Nlay
         write(23,52) alt(i),alt(i+1),
     &        total_gas_gas_budget(i),
     &        total_gas_gas_budget(i)/(alt(i+1)-alt(i))
      enddo ! i
      close(23)
      write(*,*) '---'
      write(*,*) 'Output file was generated:'
      write(*,*) res_file(1:strlen(res_file))
c
      res_file='./results/gas_gas_coolingrate.txt'
      line='alt. inf. (m) / alt. sup. (m) /'
     &     //' cooling rate [K/(time_period)] '
     &     //'(positive means heating)'
      open(23,file=res_file(1:strlen(res_file)))
      write(23,10) line(1:strlen(line))
      do i=1,Nlay
         write(23,52) alt(i),alt(i+1),
     &        total_gas_gas_budget(i)
     &        /(alt(i+1)-alt(i))/capa(i)
      enddo ! i
      close(23)
      write(*,*) '---'
      write(*,*) 'Output file was generated:'
      write(*,*) res_file(1:strlen(res_file))

      return
      end


      
      subroutine record_opacity(pass,Nk,Nlay,nu,alt,tau,
     &     outfile,outfile2,descriptor,test_data)
      implicit none
      include 'max.inc'
      include 'formats.inc'
c     
c     Purpose: to record a given optical thickness array
c     
c     Input:
c       + pass: index of the pass over spectral discretization
c       + Nk: number of points for each spectrum
c       + Nlay: number of layers
c       + nu: wavenumber array [cm^-1]
c       + alt: altitude array [m]
c       + tau: array to initialize
c       + outfile: file to use for the record of optical thcknesses per layer
c       + outfile2: file to use for the record optical thicknesses integrated over the column
c       + descriptor: character string used to describe the data (used in the file)
c       + test_data: if true, every recorded variable will be tested against NaN and INF
c     
c     Output:
c       + the required "outfile" file
c     
c     I/O
      integer pass,Nk,Nlay
      double precision tau(1:Nk_mx,1:Nlay_mx)
      double precision nu(1:Nk_mx)
      double precision alt(1:Nlev_mx)
      character*(Nchar_mx) outfile,outfile2
      character*(Nchar_mx) descriptor
      logical test_data
c     temp
      integer ik,ik1,ilay
      double precision tau_integrated(1:Nk_mx)
      logical is_nan,is_inf
      character*(Nchar_mx) line
c     label
      integer strlen
      character*(Nchar_mx) label
      label='subroutine record_opacity'
c
      line='nu (cm^-1) / z (m) / '
     &     //descriptor(1:strlen(descriptor))
c      
      open(11,file=outfile(1:strlen(outfile)),access='append')
      if (pass.eq.1) then
         ik1=1
         write(11,10) line(1:strlen(line))
      else
         ik1=2
      endif
      do ik=ik1,Nk
         do ilay=1,Nlay
            if (test_data) then
c     test value against NaN
               call test_nan_dble(tau(ik,ilay),is_nan)
               call test_inf(tau(ik,ilay),is_inf)
               if ((is_nan).or.(is_inf)) then
                  call error(label)
                  write(*,*) 'tau(',ik,',',ilay,')=',tau(ik,ilay)
                  write(*,*) 'data=',descriptor(1:strlen(descriptor))
                  write(*,*) 'alt(',ilay,')=',alt(ilay)
                  write(*,*) 'alt(',ilay+1,')=',alt(ilay+1)
                  write(*,*) 'nu_ref(',ik,')=',nu(ik)
                  stop
               endif            ! is_nan
            endif               ! test_data
c
            write(11,*) nu(ik),alt(ilay),alt(ilay+1),tau(ik,ilay)
         enddo                  ! ilay
         write(11,*)
      enddo                     ! ik
      close(11)
      if (pass.eq.1) then
         write(*,*) 'Output file was generated:'
      else
         write(*,*) 'Output file was updated:'
      endif
      write(*,*) outfile(1:strlen(outfile))
c
      do ik=1,Nk
         tau_integrated(ik)=0.0D+0
         do ilay=1,Nlay
            tau_integrated(ik)=tau_integrated(ik)+tau(ik,ilay)
         enddo                  ! ilay
      enddo                     ! ik      
c     
      line='nu (cm^-1) / '
     &     //descriptor(1:strlen(descriptor))
c     
      open(12,file=outfile2(1:strlen(outfile2)),access='append')
      if (pass.eq.1) then
         ik1=1
         write(12,10) line(1:strlen(line))
      else
         ik1=2
      endif
      do ik=ik1,Nk
         write(12,*) nu(ik),tau_integrated(ik)
      enddo                     ! ik
      close(12)
      if (pass.eq.1) then
         write(*,*) 'Output file was generated:'
      else
         write(*,*) 'Output file was updated:'
      endif
      write(*,*) outfile2(1:strlen(outfile2))
      
      return
      end
      
