c     Copyright (C)
c     2014-2018 Centre National de la Recherche Scientifique
c     2014-2018 Mesotar
c     
c     This file must be used under the terms of the CeCILL license.
c     This source file is licensed as described in the file COPYING, which
c     you should have received as part of this distribution.  The terms
c     are also available at
c     http://www.cecill.info/licences/Licence_CeCILL_V2.1-en.txt
c
      subroutine tau_from_zk(z1,z2,k1,k2,tau)
      implicit none
      include 'max.inc'
c
c     Purpose: compute the optical thickness between two levels
c     where altitude and k are known
c
c     Input:
c       + z1: altitude of the first level (m)
c       + z2: altitude of the second level (m), z2>z1
c       + k1: absorption coefficient at the first level (m^-1)
c       + k2: absorption coefficient at the second level (m^-1)
c
c     Output:
c       + tau: absorption optical thickness between (z1,k1) and (z2,k2)
c
c     I/O
      double precision z1,z2
      double precision k1,k2
      double precision tau
c     temp
c     label
      integer strlen
      character*(Nchar_mx) label
      label='subroutine tau_from_zk'

      if (z2.lt.z1) then
         call error(label)
         write(*,*) 'z1=',z1
         write(*,*) 'z2=',z2
         write(*,*) 'z2 < z1'
         stop
      endif

      if (z1.eq.z2) then
         tau=0.0D+0
      else if ((k1.le.0.0D+0).or.(k2.le.0.0D+0)) then
c     linear evolution of k(z)
         tau=(k1+k2)*(z2-z1)/2.0D+0
      else if (k1.eq.k2) then
         tau=k1*(z2-z1)
      else
c     exponential evolution of k(z)
         tau=(z2-z1)*(k1-k2)/dlog(k1/k2)
      endif

      return
      end
